/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */

package org.yajre;

import org.yajre.reteoo.TupleSource;
import org.yajre.reteoo.ReteTuple;
import org.yajre.reteoo.TupleSink;

import org.yajre.spi.Declaration;

/**
 * A debug listener can be set on a Working Memory user session to be
 * notified of each step in the rule matching chain.
 * 
 * <p> At each step, the source will be identical to the previous
 * sink, or a new descent has begun.  The sink will be null at the end
 * of the chain.
 * 
 * @author John Pritchard
 */
public interface ChainListener {

    /**
     * The implementor of this method <b>must never</b> throw an
     * exception.
     *
     * @param src Current node.
     *
     * @param tuple Current assertion.
     *
     * @param sink Next node.
     */
    public void chainStepAssertion( TupleSource src, ReteTuple tuple, TupleSink sink);

    /**
     * The implementor of this method <b>must never</b> throw an
     * exception.
     *
     * @param src Current node.
     *
     * @param decl Current retraction.
     *
     * @param sink Next node.
     */
    public void chainStepRetraction( TupleSource src, Declaration decl, TupleSink sink);
}
