/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre;

import java.io.PrintStream;
import java.io.PrintWriter;

import java.util.Locale;
import java.util.ResourceBundle;

/**
 * Base exception and exception wrapper.  As an exception wrapper it
 * behaves as the wrapped exception.  
 *
 * @author <a href="mailto:bob@werken.com">bob@werken.com</a>
 * @author John Pritchard (john@syntelos.org)
 */
public class DroolsException extends IllegalStateException {

    private Throwable source;

    public DroolsException(){
	super();
    }

    public DroolsException(String msg){
	super(msg);
    }

    public DroolsException(Throwable t){
	super();

        this.source = t;
    }

    /**
     * This is useful only if the object itself is needed.  Otherwise,
     * this wrapper supports the message, string and output functions
     * of the wrapped object.
     */
    public final Throwable getWrapped(){ return this.source;}

    public final String getMessage(){

	if ( null != this.source)

	    return this.source.getMessage();
	else
	    return super.getMessage();
    }

    public final String getLocalizedMessage(){

	if ( null != this.source)

	    return this.source.getLocalizedMessage();
	else
	    return super.getLocalizedMessage();
    }

    public final void printStackTrace(){

	if ( null != this.source)
	    this.source.printStackTrace();
	else
	    super.printStackTrace();
    }

    public final void printStackTrace(PrintStream s){

	if ( null != this.source)
	    this.source.printStackTrace(s);
	else
	    super.printStackTrace(s);
    }

    public final void printStackTrace(PrintWriter s){

	if ( null != this.source)
	    this.source.printStackTrace(s);
	else
	    super.printStackTrace(s);
    }

    public final String toString(){

	if ( null != this.source)

	    return this.source.toString();
	else
	    return super.toString();
    }

    public final boolean equals( Object ano){

	if ( null != this.source)

	    return this.source.equals(ano);
	else
	    return super.equals(ano);
    }

    public final int hashCode(){

	if ( null != this.source)

	    return this.source.hashCode();
	else
	    return super.hashCode();
    }
}
