/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre.reteoo;

import org.yajre.WorkingMemory;

import org.yajre.spi.Action;
import org.yajre.spi.Declaration;
import org.yajre.spi.ActionInvokationException;

import java.util.LinkedList;
import java.util.Iterator;

/**
 * 
 */
public class Agenda {

    private WorkingMemory workingMemory;

    private final LinkedList items = new LinkedList();

    public Agenda(WorkingMemory workingMemory){

        this.workingMemory = workingMemory;
    }

    /**
     * Deallocate resources.
     */
    public void destroy(){

	this.workingMemory = null;

	this.items.clear();
    }

    final void addToAgenda(ReteTuple tuple, Action action){

        if ( action == null )

            return;
	else 
	    this.items.add( new AgendaItem( tuple, action));
    }
    
//     public final void retract(Declaration decl){

//         Iterator   itemIter = this.items.iterator();

//         AgendaItem eachItem = null;

//         while ( itemIter.hasNext()){

//             eachItem = (AgendaItem) itemIter.next();

//             if ( eachItem.containsRootFactObject( object ) )

//                 itemIter.remove();

//         }
//     }

    public final boolean isEmpty(){

        return this.items.isEmpty();
    }

    public final void fireNextItem() throws ActionInvokationException {

        if ( this.items.isEmpty()){

            return;
        }
	else {
	    AgendaItem item = (AgendaItem) this.items.removeFirst();

	    item.fire( this.workingMemory );
	}
    }

    public final void fireAll() throws ActionInvokationException {

        while (! this.items.isEmpty()){

	    AgendaItem item = (AgendaItem) this.items.removeFirst();

	    item.fire( this.workingMemory );
	}
    }

    /**
     * If the agenda is empty, return null.  Otherwise a list of rule
     * names for which we have actions.
     */
    public final String[] ruleNames(){

	int many = this.items.size();

	if ( 1 > many)

	    return null;

	else {
	    String[] ret = new String[many];

	    AgendaItem[] list = new AgendaItem[many];

	    this.items.toArray(list);

	    for ( int cc = 0; cc < many; cc++){

		ret[cc] = list[cc].getRuleName();
	    }
	    return ret;
	}
    }
}

final class AgendaItem {

    private ReteTuple tuple;

    private Action    action;
    
    AgendaItem(ReteTuple tuple, Action action){

        this.tuple   = tuple;

        this.action = action;
    }

    boolean containsRootFactObject(Object object){

        return getTuple().containsRootFactObject( object );
    }

    ReteTuple getTuple(){

        return this.tuple;
    }
    
    Action getAction(){

        return this.action;
    }

    String getRuleName(){

        return this.action.getRuleName();
    }

    void fire( WorkingMemory mem) throws ActionInvokationException {

        getAction().invoke( getTuple(), mem);
    }
}
