/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre.reteoo;

import org.yajre.WorkingMemory;
import org.yajre.AssertionException;
import org.yajre.RetractionException;
import org.yajre.spi.Declaration;
import org.yajre.spi.FactExtractor;

import java.util.Set;
import java.util.HashSet;

/**
 * <i>Consistent-Assignment</i> node in the Rete-OO network.
 *
 * @author <a href="mailto:bob@werken.com">bob@werken.com</a>
 * @author John Pritchard (john@syntelos.com)
 */
public final class AssignmentNode extends TupleSource implements TupleSink {

    private Set           tupleDeclarations;

    private Declaration   targetDeclaration;

    private FactExtractor factExtractor;

    public AssignmentNode(TupleSource src, Declaration decl, FactExtractor ext){

        this.factExtractor     = ext;

        this.targetDeclaration = decl;

        this.tupleDeclarations = new HashSet(src.getTupleDeclarations());

        this.tupleDeclarations.add(decl);
    }

    public Set getTupleDeclarations(){

        return this.tupleDeclarations;
    }

    /**
     * Retrieve the {@link Declaration} which is the target of the
     * assignment.
     *
     * @return The target <code>Declaration</code>.
     */
    public Declaration getTargetDeclaration(){

        return this.targetDeclaration;
    }

    /** 
     * Retrieve the {@link FactExtractor} used to generate the
     * right-hand-side value for the assignment.
     *
     * @return The <code>FactExtrator</code>.
     */
    public FactExtractor getFactExtractor(){

        return this.factExtractor;
    }

    public void sinkAssertTuple(TupleSource src, ReteTuple tuple, WorkingMemory mem) throws AssertionException {

        Object fact_value = getFactExtractor().extractFact( tuple, mem);

        ReteTuple newTuple = new ReteTuple();

        newTuple.putAll(tuple);

        newTuple.put( getTargetDeclaration(), fact_value);

        newTuple.addAllRootFactObjects( tuple.getRootFactObjects());

        propagateAssertion( newTuple, mem);
    }

    public void sinkRetractObject(TupleSource src, Declaration decl, WorkingMemory mem) throws RetractionException {

        propagateRetraction( decl, mem);
    }

    public String toString(){

	return "AssignmentNode[decls="+this.tupleDeclarations+", target="+this.targetDeclaration+", extractor="+this.factExtractor+"]";
    }
}
