/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre.reteoo;

import org.yajre.spi.Declaration;

import java.util.Set;
import java.util.HashSet;

/**
 * Implementation of {@link ReteTuple} with a single column, based
 * upon a <i>root fact object parameter</i> of a rule.
 *
 * @author <a href="mailto:bob@werken.com">bob@werken.com</a>
 * @author John Pritchard (john@syntelos.org)
 */
public final class ParameterTuple extends ReteTuple {

    /**
     * The parameter <code>Declaration</code>. 
     */
    private Declaration declaration;

    /**
     * The object bound to the parameter. 
     */
    private Object object;

    /** 
     *  @param declaration The parameter <code>Declaration</code>.
     *
     *  @param object The object bound to the <code>Declaration</code>.
     */
    public ParameterTuple(Declaration declaration, Object object){

        this.declaration = declaration;

        this.object      = object;

        addRootFactObject(object);
    }

    /**
     * @return The current value of the parameter.
     */
    public Object getParameterObject(){

        return this.object;
    }

    /**
     * @return The fact identity.
     */
    public Declaration getParameterDeclaration(){

        return this.declaration;
    }

    /**
     * @return The fact value given its declaration.
     */
    public Object get(Declaration decl){

        if ( this.declaration.equals( decl ) )

            return getParameterObject();
	else
	    return super.get( decl );
    }

    /**
     * @return Known declarations.
     */
    public Set getDeclarations(){

        Set decls = new HashSet();

        decls.add( getParameterDeclaration() );

        decls.addAll( super.getDeclarations() );

        return decls;
    }

    public String toString(){
	return "ParameterTuple[tuple="+super.getDeclarations()+", facts="+super.getRootFactObjects()+", decl="+this.declaration+", object="+this.object+"]";
    }
}
