/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre.reteoo;

import org.yajre.spi.Tuple;
import org.yajre.spi.Declaration;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 * Base Rete-OO {@link Tuple} implementation.
 *
 * @see Tuple
 *  
 * @author <a href="mailto:bob@werken.com">bob@werken.com</a>
 * @author John Pritchard (john@syntelos.com)
 */
public class ReteTuple implements Tuple {

    private final Map tuple = new HashMap();

    private final Set rootFactObjects = new HashSet();

    /** 
     */
    public ReteTuple(){
	super();
    }

    protected final void addRootFactObject(Object object){

        this.rootFactObjects.add( object );
    }

    protected final void addAllRootFactObjects(Set objects){

        this.rootFactObjects.addAll( objects );
    }

    protected final Set getRootFactObjects(){

        return this.rootFactObjects;
    }

    protected final boolean containsRootFactObject(Object object){

        return this.rootFactObjects.contains( object );
    }

    /**
     * Set the value of this variable.
     */
    public final void put(Declaration declaration, Object value){

        this.tuple.put( declaration, value );
    }

    /**
     * @returns The value of this variable.
     */
    public Object get(Declaration declaration){

        return this.tuple.get( declaration );
    }

    /**
     * @returns If this tuple includes this variable.
     */
    public boolean contains(Declaration declaration){

        return this.tuple.containsKey( declaration );
    }

    public final void putAll( Tuple tuple){

        Iterator decls = tuple.getDeclarations().iterator();

        Declaration decl = null;

        while ( decls.hasNext()){

            decl = (Declaration)decls.next();

            put( decl, tuple.get(decl));
        }
    }

    /**
     * @returns The set of variable declarations.
     */
    public Set getDeclarations(){

        return tuple.keySet();
    }

    public String toString(){

	String clan = this.getClass().getName();

	int idx = clan.lastIndexOf('.');
	if ( 0 < idx)
	    clan = clan.substring(idx+1);

        return clan+"[tuple="+this.tuple+", facts="+this.rootFactObjects+"]";
    }

    public boolean equals ( Object ano){

	if ( ano instanceof ReteTuple)

	    return this.tuple.equals( ((ReteTuple)ano).tuple);
	else
	    return false;
    }

    public int hashCode(){

	return this.tuple.hashCode();
    }
}
