/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre.semantic.java;

import org.yajre.WorkingMemory;

import org.yajre.spi.FactExtractor;
import org.yajre.spi.FactExtractionException;
import org.yajre.spi.Tuple;
import org.yajre.spi.Declaration;

import bsh.Interpreter;
import bsh.EvalError;

import java.util.Set;
import java.util.Iterator;

/**
 * A {@link FactExtractor} using <a href="http://beanshell.org/">BeanShell</a>
 * to perform run-time fact extraction.
 *
 * @author <a href="mailto:bob@werken.com">bob@werken.com</a>
 * @author John Pritchard (john@syntelos.org)
 */
public final class BeanShellFactExtractor implements FactExtractor {

    /**
     * The expression to extract the fact. 
     */
    private String        expression;

    /**
     * The members referenced by the expression. 
     */
    private Declaration[] requiredTupleMembers;

    /** 
     * Construct.
     *
     * @param expr The fact extraction expression.
     * @param members Set of variables referenced.
     */
    public BeanShellFactExtractor(String expr, Set members){
	super();

        this.expression = expr;

	int many = members.size();

        this.requiredTupleMembers = new Declaration[many];

        members.toArray(this.requiredTupleMembers);
    }

    public Declaration[] getRequiredTupleMembers(){

        return this.requiredTupleMembers;
    }

    /**
     * Retrieve the fact-extraction BeanShell expression.
     *
     * @return The fact-extraction expression.
     */
    public String getExpression(){

        return this.expression;
    }

    public Object extractFact(Tuple tup, WorkingMemory mem) throws FactExtractionException {

	Interpreter interp = BeanShellUtil.interp(mem);

        try {

	    BeanShellUtil.cleanUpInterpreter( interp, tup);

            BeanShellUtil.setUpInterpreter( interp, tup);
            
            return interp.eval( getExpression());
        }
        catch (EvalError e){

            throw new FactExtractionException(e);
        }
    }

    public String toString(){
	return "BeanShellFactExtractor[required="+BeanShellUtil.ToString(this.requiredTupleMembers)+"]";
    }
}
