/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre.semantic.java;

import org.yajre.WorkingMemory;

import org.yajre.spi.FilterCondition;
import org.yajre.spi.FilterException;
import org.yajre.spi.Declaration;
import org.yajre.spi.Tuple;

import bsh.Interpreter;
import bsh.EvalError;

import java.util.Set;
import java.util.Iterator;

/**
 * {@link FilterCondition} using <a
 * href="http://beanshell.org/">BeanShell</a> for evaluation.
 *
 * @author <a href="mailto:bob@werken.com">bob@werken.com</a>
 * @author John Pritchard (john@syntelos.org)
 */
public final class BeanShellFilterCondition implements FilterCondition {

    /** The BeanShell expression. */
    private String        filterExpr;

    /** Variables referenced by the expression. */
    private Declaration[] requiredTupleMembers;

    /** 
     * The expression used to construct this <b>must</b> produce a
     * <b>boolean</b> result, else a {@link
     * NonBooleanExpressionException} will be thrown at run-time.
     *
     * @param expr A boolean condition expression.
     *
     * @param members Set of variables referenced.
     */
    public BeanShellFilterCondition(String expr, Set members){

        this.filterExpr = expr.trim();

        this.requiredTupleMembers = new Declaration[members.size()];

	members.toArray(this.requiredTupleMembers);
    }

    /** Retrieve the BeanShell filtering expression.
     *
     *  @return The filtering expression.
     */
    public String getFilterExpr()
    {
        return this.filterExpr;
    }

    public Declaration[] getRequiredTupleMembers()
    {
        return this.requiredTupleMembers;
    }

    public boolean isAllowed(Tuple tuple, WorkingMemory session) throws FilterException
    {
        boolean result = false;

	Interpreter interp = BeanShellUtil.interp( session );

        try
        {
	    BeanShellUtil.cleanUpInterpreter( interp,
					      tuple );

            BeanShellUtil.setUpInterpreter( interp,
                                            tuple );

            Object resultObj = interp.eval( getFilterExpr() );
            
            if ( resultObj instanceof Boolean )
            {
                result = ((Boolean)resultObj).booleanValue();
            }
            else
            {
//                 try
//                 {
//                     BeanShellUtil.cleanUpInterpreter( interp,
//                                                       tuple );
//                 }
//                 catch (EvalError e)
//                 {
//                     initializeInterpreter();
//                     // no further throws, since we have a higher-priority
//                     // exception regarding the non-Boolean return from bsh.
//                 }

                throw new NonBooleanExpressionException();
            }
        }
        catch (EvalError e)
        {
	    ///initializeInterpreter();

            throw new FilterException( e );
        }

        return result;
    }

    public String toString(){
	return "BeanShellFilterCondition[expr=`"+this.filterExpr+"', required="+BeanShellUtil.ToString(this.requiredTupleMembers)+"]";
    }
    
}
