/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre.semantic.java;

import org.yajre.WorkingMemory;

import org.yajre.spi.Tuple;
import org.yajre.spi.Declaration;

import bsh.Interpreter;
import bsh.EvalError;

import java.util.Set;
import java.util.Iterator;

/**
 * Utilities for merging the Tuple into a form usable by <a
 * href="http://beanshell.org/">BeanShell.</a>
 *
 * @author <a href="mailto:bob@werken.com">bob@werken.com</a>
 * @author John Pritchard (john@syntelos.org)
 */
public abstract class BeanShellUtil {

    /**
     * Working memory property for our Bean Shell interpreter.
     */
    public final static String SESS_INTERP = "org.yajre.semantic.java.bsh.Interpreter";
    /**
     * Rule base attribute for Bean Shell (Java) import statements in
     * a possibly null string array.
     */
    public final static String RBAT_IMPORT = "org.yajre.semantic.java.imports";

    /** 
     * Get or create the BeanShell interpreter.
     */
    public final static Interpreter interp( WorkingMemory session){ //(jdp)

	Interpreter interp = (Interpreter)session.getProperty(SESS_INTERP);

	if ( null == interp){

	    interp = new Interpreter();

	    String[] imports = (String[])session.getRuleBaseAttribute(RBAT_IMPORT);

	    if ( null != imports){

		int many = imports.length;

		for ( int cc = 0; cc < many; cc++){
		    try {
			interp.eval(imports[cc]);
		    }
		    catch ( EvalError ever){

			synchronized(System.err){

			    System.err.println("BBBUGGGG! Imports evaluation error in:\n>\n>\t"+imports[cc]+"\n>\n>\t");
			    ever.printStackTrace();
			}
		    }
		}
	    }

	    session.setProperty(SESS_INTERP,interp);

	    return interp;
	}
	else
	    return interp;
    }
    
    /** Set up the interpreter with bindings to the
     *  members of the tuple.
     *
     *  @param interp The interpreter to configure.
     *  @param tuple The tuple to map into the interpreter's namespace.
     */
    public static void setUpInterpreter(Interpreter interp,
                                        Tuple tuple) throws EvalError
    {
        Set         decls    = tuple.getDeclarations();

        Iterator    declIter = decls.iterator();
        Declaration eachDecl = null;

        while ( declIter.hasNext() )
        {
            eachDecl = (Declaration) declIter.next();
            
            interp.set( eachDecl.getIdentifier(),
                        tuple.get( eachDecl ) );
        }
    }

    /** Clean up the interpreter's bindings to the
     *  members of the tuple.
     *
     *  @param interp The interpreter to clean up.
     *  @param tuple The tuple to unmap from the interpreter's namespace.
     */
    public static void cleanUpInterpreter(Interpreter interp,
                                          Tuple tuple) throws EvalError
    {
        Set         decls    = tuple.getDeclarations();

        Iterator    declIter = decls.iterator();
        Declaration eachDecl = null;

        while ( declIter.hasNext() )
        {
            eachDecl = (Declaration) declIter.next();
            
            interp.unset( eachDecl.getIdentifier() );
        }
    }

    public final static String ToString( Object[] ary){
	StringBuffer strbuf = new StringBuffer();

	strbuf.append('{');

	if ( null != ary){
	    int many = ary.length;

	    for ( int cc = 0; cc < many; cc++){
		if ( 0 < cc)
		    strbuf.append(',');

		strbuf.append(ary[cc]);
	    }
	}
	strbuf.append('}');
	return strbuf.toString();
    }
}
