/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */

package org.yajre.semantic.java;

import bsh.* ;

import java.io.* ;

import java.util.* ;

/**
 * Test bsh interpreter with integrated rules impl tester.
 * 
 * @author John Pritchard
 */
public class bshi extends Object {

    private final static String prompt = "bshi> ";

  //

    public final Interpreter interp = new Interpreter();

    public final test tt = new test();

    public bshi(){
	super();
    }

    public final void init(){

	if ( tt.active()){

	    tt.interp = this.interp;

	    tt.init();
	}
    }

    public final Object eval( String input, PrintStream out) throws EvalError {

	Object value = this.interp.eval(input);

	if ( null != value)
	    out.println(value);
	else
	    out.println();

	return value;
    }

  //

    public final void console ( DataInputStream din, PrintStream out, PrintStream err) throws IOException {

	String line, name;

	Object value;

	int linelen;

	StringTokenizer strtok;

	err.print(prompt);

	while (null != (line = din.readLine())){

	    try {

		linelen = line.length();

		if ( 0 < linelen){

		    switch(line.charAt(0)){

			/*
			 * Get <name>
			 */
		    case 'g':
			if ( 1 < linelen && ' ' == line.charAt(1)){
			    strtok = new StringTokenizer(line.substring(1)," ");

			    if (strtok.hasMoreTokens()){

				name = strtok.nextToken();

				value = interp.get(name);

				out.println(value);

				break;
			    }
			    else
				throw new IllegalArgumentException("Command 'g'et requires argument `name'.");
			}
			else
			    eval(line,out);

			break;

			/*
			 * Set <name> <value>
			 */
		    case 's':
			if ( 1 < linelen && ' ' == line.charAt(1)){
			    strtok = new StringTokenizer(line.substring(1)," ");

			    if (strtok.hasMoreTokens()){

				name = strtok.nextToken();

				if (strtok.hasMoreTokens()){

				    value = strtok.nextToken();

				    interp.set(name,value);

				    out.println(value);

				    break;
				}
			    }

			    throw new IllegalArgumentException("Command 's'et requires arguments `name' and `value'.");
			}
			else
			    eval(line,out);

			break;

			/*
			 * Unset <name>
			 */
		    case 'u':
			if ( 1 < linelen && ' ' == line.charAt(1)){
			    strtok = new StringTokenizer(line.substring(1)," ");

			    if (strtok.hasMoreTokens()){

				name = strtok.nextToken();

				interp.unset(name);

				out.println();

				break;
			    }
			    else
				throw new IllegalArgumentException("Command 'u'nset requires argument `name'.");
			}
			else
			    eval(line,out);

			break;

		    case '?':
		    case 'h':

			if ( 1 == linelen)

			    help(out);
			else
			    this.eval(line,out);

			break;

		    case 'r':

			if ( 1 == linelen){

			    if ( tt.active())

				this.tt.console(din,out,err);
			    else
				err.println("Rules interpreter not configured.");
			}
			else
			    this.eval(line,out);

			break;

		    case 'q':

			if ( 1 == linelen)

			    return;
			else
			    this.eval(line,out);

			break;

		    default:

			this.eval(line,out);

			break;
		    }
		}
	    }
	    catch ( IllegalArgumentException ilarg){

		String msg = ilarg.getMessage();

		if ( null == msg)
		    help(out);
		else
		    out.println("Error: "+msg);

	    }
	    catch ( EvalError ever){

		String msg = ever.getMessage();
		try {
		    int idx = msg.indexOf('>');

		    if ( 0 < idx)
			msg = msg.substring(idx+3).trim();

		} catch ( Exception x){}

		out.println("[error] "+msg);
	    }
	    catch ( Exception exc){

		out.print("[error] ");

		exc.printStackTrace(out);
	    }

	    err.print(prompt);
	}
    }

  //

    public final static void usage( PrintStream out){
	out.println();
	out.println(" Usage: bshi [-r <rules>]");
	out.println();
	out.println("   Test semantics of bean shell name space");
	out.println("   with optional rules engine.");
	out.println();
	out.println("   Rules engine:");
	out.println();
	out.println("   -r rules       -- Rule set reference for `init'.");
	out.println();
	out.println("   Enter '?' into stdin for commands' help.");
	out.println();
    }
    public final static void help( PrintStream out){
	out.println();
	out.println(" [Command]            -- [Description]");
	out.println();
	out.println(" g <name>             -- Get name, print value.");
	out.println();
	out.println(" s <name> <value>     -- Set name, print value.");
	out.println();
	out.println(" u <name>             -- Unset name.");
	out.println();
	out.println(" r                    -- Switch to rules context, see '?'.");
	out.println();
	out.println(" (eval)               -- Evaluate `bsh' input.");
	out.println();
	out.println(" q                    -- Quit (exit).");
	out.println();
	out.println(" ?|h                  -- This `help' message.");
	out.println();
    }

    public static void main(String[] argv){

	bshi bb = new bshi();

	try {
	    if ( null != argv){

		int alen = argv.length;

		if ( 0 < alen){

		    String arg;

		    for ( int argc = 0; argc < alen; argc++){
			arg = argv[argc];
			switch(arg.charAt(0)){

			case '-':
			case '/':
			    if ( 1 < arg.length()){
				switch(arg.charAt(1)){

				case 'r':
				    argc += 1;
				    if ( argc < alen){

					arg = argv[argc];

					bb.tt.ruleset(arg,System.err);

					break;
				    }
				    else
					throw new IllegalArgumentException("Option `-r' requires argument.");

				default:
				    throw new IllegalArgumentException();
				}
			    }
			    else
				throw new IllegalArgumentException();
			    break;
			default:
			    throw new IllegalArgumentException();
			}
		    }
		}
	    }
	}
	catch ( IllegalArgumentException ilarg){
	    String msg = ilarg.getMessage();
	    if ( null == msg)
		usage(System.out);
	    else
		System.err.println("Error: "+msg);
	    System.exit(1);
	}

	try {

	    bb.init();

	    DataInputStream din = new DataInputStream(System.in);

	    bb.console(din,System.out,System.err);

	    System.err.println("bye!");

	    System.exit(0);
	}
	catch ( Exception exc){
	    exc.printStackTrace();
	    System.exit(1);
	}
    }
}
