/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
/*
 * LICENSE.txt,v 1.1 2001/12/12 19:33:28 werken 
 *
 * Copyright 2001 (C) The Werken Company. All Rights Reserved.
 * 
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 * 
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. The name "Drools" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of The Werken Company.  For written permission,
 *    please contact bob@werken.com.
 * 
 * 4. Products derived from this Software may not be called "Drools"
 *    nor may "Drools" appear in their names without prior written
 *    permission of The Werken Company. Drools is a registered
 *    trademark of The Werken Company.
 * 
 * 5. Due credit should be given to the Drools Project
 *    (http://drools.org/).
 * 
 * THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package org.yajre.semantic.java.io;

import org.yajre.semantic.java.JavaObjectType;
import org.yajre.semantic.java.BeanShellFactExtractor;
import org.yajre.semantic.java.BeanShellFilterCondition;

import org.yajre.semantic.java.parser.JavaLexer;
import org.yajre.semantic.java.parser.JavaRecognizer;
import org.yajre.semantic.java.parser.JavaTreeParser;
import org.yajre.semantic.java.parser.JavaTokenTypes;

import org.yajre.spi.Declaration;
import org.yajre.spi.AssignmentCondition;

import org.dom4j.Element;
import org.dom4j.ElementPath;

import antlr.TokenStreamException;
import antlr.RecognitionException;
import antlr.collections.AST;

import java.io.StringReader;

import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

class CondHandler extends BaseRuleSetHandler
{
    CondHandler(RuleSetReader reader)
    {
        super( reader );
    }

    public void onEnd(ElementPath path)
    {
        Element elem = path.getCurrent();

        String expr = elem.getTextTrim();

        JavaLexer lexer = new JavaLexer( new StringReader( expr ) );
        JavaRecognizer parser = new JavaRecognizer( lexer );

        try
        {
            parser.ruleCondition();

            AST ast = parser.getAST();

            analyze( expr, ast );
        }
        catch (TokenStreamException e)
        {

	    System.err.println(">In "+getReader().getCurrentRule());
	    System.err.print(">  ");

            e.printStackTrace();

	    throw new IllegalStateException(e.getMessage());
        }
        catch (RecognitionException e)
        {

	    System.err.println(">In "+getReader().getCurrentRule());
	    System.err.print(">  ");

            e.printStackTrace();

	    throw new IllegalStateException(e.getMessage());
        }
    }

    private void analyze(String expr,
                         AST ast) throws RecognitionException
    {
        if ( ast.getType() == JavaTokenTypes.ASSIGN )
        {
            integrateAssignmentCondition( expr,
                                          ast );
        }
        else
        {
            integrateFilterCondition( expr,
                                      ast );
        }
    }

    private void integrateFilterCondition(String expr,
                                          AST ast) throws RecognitionException
    {
        JavaTreeParser treeParser = new JavaTreeParser();

        treeParser.init();

        treeParser.filterCondition( ast );

        List refs = treeParser.getVariableReferences();

        Set         decls    = new HashSet();
        Declaration eachDecl = null;

        Iterator    refIter = refs.iterator();
        String      eachRef = null;

        while ( refIter.hasNext() )
        {
            eachRef = (String) refIter.next();

            eachDecl = getReader().getDeclaration( eachRef );

            if ( eachDecl != null )
            {
                decls.add( eachDecl );
            }
        }

        BeanShellFilterCondition filterCond = new BeanShellFilterCondition( expr,
                                                                            decls );

        getReader().getCurrentRule().addFilterCondition( filterCond );
    }

    private void integrateAssignmentCondition(String expr,
                                              AST ast) throws RecognitionException
    {
        JavaTreeParser treeParser = new JavaTreeParser();

        treeParser.init();

        treeParser.assignmentCondition( ast );

        List refs = treeParser.getVariableReferences();

        String      identifier = (String) refs.get( 0 );
        Declaration targetDecl = getReader().getDeclaration( identifier );

        Set         rhsRefs  = new HashSet();
        String      eachRef  = null;
        Declaration eachDecl = null;

        for ( int i = 1 ; i < refs.size() ; ++i )
        {
            eachRef = (String) refs.get( i );
            eachDecl = getReader().getDeclaration( eachRef );

            if ( eachDecl != null )
            {
                rhsRefs.add( eachDecl );
            }
        }

        BeanShellFactExtractor extractor = new BeanShellFactExtractor( expr,
                                                                       rhsRefs );

        AssignmentCondition assignCond = new AssignmentCondition( targetDecl,
                                                                  extractor );

        getReader().getCurrentRule().addAssignmentCondition( assignCond );
    }
}

