/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */
// $ANTLR 2.7.1: "java.g" -> "JavaRecognizer.java"$

package org.yajre.semantic.java.parser;

import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;
import antlr.collections.AST;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

/** Java 1.3 Recognizer.
 *
 *  This grammar is in the PUBLIC DOMAIN
 * 
 *	@author John Mitchell		johnm@non.net
 *	@author Terence Parr		parrt@magelang.com
 *	@author John Lilley			jlilley@empathy.com
 *	@author Scott Stanchfield	thetick@magelang.com
 *	@author Markus Mohnen       mohnen@informatik.rwth-aachen.de
 *  @author Peter Williams      pete.williams@sun.com
 *  @author Allan Jacobs        Allan.Jacobs@eng.sun.com
 *  @author Steve Messick       messick@redhills.com
 *
 */
public class JavaRecognizer extends antlr.LLkParser
       implements JavaTokenTypes
 {

protected JavaRecognizer(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public JavaRecognizer(TokenBuffer tokenBuf) {
  this(tokenBuf,2);
}

protected JavaRecognizer(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public JavaRecognizer(TokenStream lexer) {
  this(lexer,2);
}

public JavaRecognizer(ParserSharedInputState state) {
  super(state,2);
  tokenNames = _tokenNames;
}

	public final void compilationUnit() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST compilationUnit_AST = null;
		
		{
		switch ( LA(1)) {
		case LITERAL_package:
		{
			packageDefinition();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case EOF:
		case FINAL:
		case ABSTRACT:
		case STRICTFP:
		case SEMI:
		case LITERAL_import:
		case LITERAL_private:
		case LITERAL_public:
		case LITERAL_protected:
		case LITERAL_static:
		case LITERAL_transient:
		case LITERAL_native:
		case LITERAL_threadsafe:
		case LITERAL_synchronized:
		case LITERAL_volatile:
		case LITERAL_class:
		case LITERAL_interface:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		_loop4:
		do {
			if ((LA(1)==LITERAL_import)) {
				importDefinition();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop4;
			}
			
		} while (true);
		}
		{
		_loop6:
		do {
			if ((_tokenSet_0.member(LA(1)))) {
				typeDefinition();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop6;
			}
			
		} while (true);
		}
		AST tmp1_AST = null;
		tmp1_AST = (AST)astFactory.create(LT(1));
		match(Token.EOF_TYPE);
		compilationUnit_AST = (AST)currentAST.root;
		returnAST = compilationUnit_AST;
	}
	
	public final void packageDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST packageDefinition_AST = null;
		Token  p = null;
		AST p_AST = null;
		
		try {      // for error handling
			p = LT(1);
			if (inputState.guessing==0) {
				p_AST = (AST)astFactory.create(p);
				astFactory.makeASTRoot(currentAST, p_AST);
			}
			match(LITERAL_package);
			if ( inputState.guessing==0 ) {
				p_AST.setType(PACKAGE_DEF);
			}
			identifier();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp2_AST = null;
			tmp2_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			packageDefinition_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				consume();
				consumeUntil(_tokenSet_1);
			} else {
			  throw ex;
			}
		}
		returnAST = packageDefinition_AST;
	}
	
	public final void importDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST importDefinition_AST = null;
		Token  i = null;
		AST i_AST = null;
		
		try {      // for error handling
			i = LT(1);
			if (inputState.guessing==0) {
				i_AST = (AST)astFactory.create(i);
				astFactory.makeASTRoot(currentAST, i_AST);
			}
			match(LITERAL_import);
			if ( inputState.guessing==0 ) {
				i_AST.setType(IMPORT);
			}
			identifierStar();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp3_AST = null;
			tmp3_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			importDefinition_AST = (AST)currentAST.root;
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				consume();
				consumeUntil(_tokenSet_1);
			} else {
			  throw ex;
			}
		}
		returnAST = importDefinition_AST;
	}
	
	public final void typeDefinition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST typeDefinition_AST = null;
		AST m_AST = null;
		
		try {      // for error handling
			switch ( LA(1)) {
			case FINAL:
			case ABSTRACT:
			case STRICTFP:
			case LITERAL_private:
			case LITERAL_public:
			case LITERAL_protected:
			case LITERAL_static:
			case LITERAL_transient:
			case LITERAL_native:
			case LITERAL_threadsafe:
			case LITERAL_synchronized:
			case LITERAL_volatile:
			case LITERAL_class:
			case LITERAL_interface:
			{
				modifiers();
				if (inputState.guessing==0) {
					m_AST = (AST)returnAST;
				}
				{
				switch ( LA(1)) {
				case LITERAL_class:
				{
					classDefinition(m_AST);
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
					break;
				}
				case LITERAL_interface:
				{
					interfaceDefinition(m_AST);
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				typeDefinition_AST = (AST)currentAST.root;
				break;
			}
			case SEMI:
			{
				AST tmp4_AST = null;
				tmp4_AST = (AST)astFactory.create(LT(1));
				match(SEMI);
				typeDefinition_AST = (AST)currentAST.root;
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			if (inputState.guessing==0) {
				reportError(ex);
				consume();
				consumeUntil(_tokenSet_2);
			} else {
			  throw ex;
			}
		}
		returnAST = typeDefinition_AST;
	}
	
	public final void ruleFile() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST ruleFile_AST = null;
		
		{
		switch ( LA(1)) {
		case LITERAL_package:
		{
			packageDefinition();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case RULE_SET:
		case LITERAL_import:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		{
		_loop10:
		do {
			if ((LA(1)==LITERAL_import)) {
				importDefinition();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop10;
			}
			
		} while (true);
		}
		ruleSet();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		ruleFile_AST = (AST)currentAST.root;
		returnAST = ruleFile_AST;
	}
	
	public final void ruleSet() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST ruleSet_AST = null;
		
		AST tmp5_AST = null;
		if (inputState.guessing==0) {
			tmp5_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp5_AST);
		}
		match(RULE_SET);
		AST tmp6_AST = null;
		if (inputState.guessing==0) {
			tmp6_AST = (AST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp6_AST);
		}
		match(IDENT);
		AST tmp7_AST = null;
		tmp7_AST = (AST)astFactory.create(LT(1));
		match(LCURLY);
		{
		int _cnt13=0;
		_loop13:
		do {
			if ((LA(1)==RULE)) {
				rule();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				if ( _cnt13>=1 ) { break _loop13; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt13++;
		} while (true);
		}
		AST tmp8_AST = null;
		tmp8_AST = (AST)astFactory.create(LT(1));
		match(RCURLY);
		ruleSet_AST = (AST)currentAST.root;
		returnAST = ruleSet_AST;
	}
	
	public final void rule() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST rule_AST = null;
		AST param_AST = null;
		
		AST tmp9_AST = null;
		if (inputState.guessing==0) {
			tmp9_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp9_AST);
		}
		match(RULE);
		AST tmp10_AST = null;
		if (inputState.guessing==0) {
			tmp10_AST = (AST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp10_AST);
		}
		match(IDENT);
		AST tmp11_AST = null;
		tmp11_AST = (AST)astFactory.create(LT(1));
		match(LPAREN);
		parameterDeclarationList();
		if (inputState.guessing==0) {
			param_AST = (AST)returnAST;
			astFactory.addASTChild(currentAST, returnAST);
		}
		AST tmp12_AST = null;
		tmp12_AST = (AST)astFactory.create(LT(1));
		match(RPAREN);
		AST tmp13_AST = null;
		tmp13_AST = (AST)astFactory.create(LT(1));
		match(LCURLY);
		{
		_loop16:
		do {
			if ((_tokenSet_3.member(LA(1)))) {
				parameterDeclaration();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop16;
			}
			
		} while (true);
		}
		whenBlock();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		thenBlock();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		AST tmp14_AST = null;
		tmp14_AST = (AST)astFactory.create(LT(1));
		match(RCURLY);
		rule_AST = (AST)currentAST.root;
		returnAST = rule_AST;
	}
	
	public final void parameterDeclarationList() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST parameterDeclarationList_AST = null;
		
		{
		switch ( LA(1)) {
		case FINAL:
		case IDENT:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		{
			parameterDeclaration();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			{
			_loop100:
			do {
				if ((LA(1)==COMMA)) {
					AST tmp15_AST = null;
					tmp15_AST = (AST)astFactory.create(LT(1));
					match(COMMA);
					parameterDeclaration();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else {
					break _loop100;
				}
				
			} while (true);
			}
			break;
		}
		case RPAREN:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			parameterDeclarationList_AST = (AST)currentAST.root;
			parameterDeclarationList_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(PARAMETERS,"PARAMETERS")).add(parameterDeclarationList_AST));
			currentAST.root = parameterDeclarationList_AST;
			currentAST.child = parameterDeclarationList_AST!=null &&parameterDeclarationList_AST.getFirstChild()!=null ?
				parameterDeclarationList_AST.getFirstChild() : parameterDeclarationList_AST;
			currentAST.advanceChildToEnd();
		}
		parameterDeclarationList_AST = (AST)currentAST.root;
		returnAST = parameterDeclarationList_AST;
	}
	
	public final void parameterDeclaration() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST parameterDeclaration_AST = null;
		AST pm_AST = null;
		AST t_AST = null;
		Token  id = null;
		AST id_AST = null;
		AST pd_AST = null;
		
		parameterModifier();
		if (inputState.guessing==0) {
			pm_AST = (AST)returnAST;
		}
		typeSpec(false);
		if (inputState.guessing==0) {
			t_AST = (AST)returnAST;
		}
		id = LT(1);
		if (inputState.guessing==0) {
			id_AST = (AST)astFactory.create(id);
		}
		match(IDENT);
		declaratorBrackets(t_AST);
		if (inputState.guessing==0) {
			pd_AST = (AST)returnAST;
		}
		if ( inputState.guessing==0 ) {
			parameterDeclaration_AST = (AST)currentAST.root;
			parameterDeclaration_AST = (AST)astFactory.make( (new ASTArray(4)).add((AST)astFactory.create(PARAMETER_DEF,"PARAMETER_DEF")).add(pm_AST).add((AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(TYPE,"TYPE")).add(pd_AST))).add(id_AST));
			currentAST.root = parameterDeclaration_AST;
			currentAST.child = parameterDeclaration_AST!=null &&parameterDeclaration_AST.getFirstChild()!=null ?
				parameterDeclaration_AST.getFirstChild() : parameterDeclaration_AST;
			currentAST.advanceChildToEnd();
		}
		returnAST = parameterDeclaration_AST;
	}
	
	public final void whenBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST whenBlock_AST = null;
		
		AST tmp16_AST = null;
		if (inputState.guessing==0) {
			tmp16_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp16_AST);
		}
		match(WHEN);
		AST tmp17_AST = null;
		tmp17_AST = (AST)astFactory.create(LT(1));
		match(LCURLY);
		{
		int _cnt20=0;
		_loop20:
		do {
			if ((_tokenSet_4.member(LA(1)))) {
				{
				if ((LA(1)==IDENT) && (LA(2)==ASSIGN)) {
					consistentAssignmentExpression();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else if ((_tokenSet_4.member(LA(1))) && (_tokenSet_5.member(LA(2)))) {
					inclusiveOrExpression();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else {
					throw new NoViableAltException(LT(1), getFilename());
				}
				
				}
				AST tmp18_AST = null;
				tmp18_AST = (AST)astFactory.create(LT(1));
				match(SEMI);
			}
			else {
				if ( _cnt20>=1 ) { break _loop20; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt20++;
		} while (true);
		}
		AST tmp19_AST = null;
		tmp19_AST = (AST)astFactory.create(LT(1));
		match(RCURLY);
		whenBlock_AST = (AST)currentAST.root;
		returnAST = whenBlock_AST;
	}
	
	public final void thenBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST thenBlock_AST = null;
		
		AST tmp20_AST = null;
		if (inputState.guessing==0) {
			tmp20_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp20_AST);
		}
		match(THEN);
		compoundStatement();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		thenBlock_AST = (AST)currentAST.root;
		returnAST = thenBlock_AST;
	}
	
	public final void consistentAssignmentExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST consistentAssignmentExpression_AST = null;
		
		AST tmp21_AST = null;
		if (inputState.guessing==0) {
			tmp21_AST = (AST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp21_AST);
		}
		match(IDENT);
		AST tmp22_AST = null;
		if (inputState.guessing==0) {
			tmp22_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp22_AST);
		}
		match(ASSIGN);
		inclusiveOrExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		consistentAssignmentExpression_AST = (AST)currentAST.root;
		returnAST = consistentAssignmentExpression_AST;
	}
	
	public final void inclusiveOrExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST inclusiveOrExpression_AST = null;
		
		exclusiveOrExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop154:
		do {
			if ((LA(1)==BOR)) {
				AST tmp23_AST = null;
				if (inputState.guessing==0) {
					tmp23_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp23_AST);
				}
				match(BOR);
				exclusiveOrExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop154;
			}
			
		} while (true);
		}
		inclusiveOrExpression_AST = (AST)currentAST.root;
		returnAST = inclusiveOrExpression_AST;
	}
	
	public final void compoundStatement() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST compoundStatement_AST = null;
		Token  lc = null;
		AST lc_AST = null;
		
		lc = LT(1);
		if (inputState.guessing==0) {
			lc_AST = (AST)astFactory.create(lc);
			astFactory.makeASTRoot(currentAST, lc_AST);
		}
		match(LCURLY);
		if ( inputState.guessing==0 ) {
			lc_AST.setType(SLIST);
		}
		{
		_loop106:
		do {
			if ((_tokenSet_6.member(LA(1)))) {
				statement();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop106;
			}
			
		} while (true);
		}
		AST tmp24_AST = null;
		tmp24_AST = (AST)astFactory.create(LT(1));
		match(RCURLY);
		compoundStatement_AST = (AST)currentAST.root;
		returnAST = compoundStatement_AST;
	}
	
	public final void ruleCondition() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST ruleCondition_AST = null;
		
		if ((LA(1)==IDENT) && (LA(2)==ASSIGN)) {
			consistentAssignmentExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			ruleCondition_AST = (AST)currentAST.root;
		}
		else if ((_tokenSet_4.member(LA(1))) && (_tokenSet_7.member(LA(2)))) {
			inclusiveOrExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			ruleCondition_AST = (AST)currentAST.root;
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		returnAST = ruleCondition_AST;
	}
	
	public final void identifier() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST identifier_AST = null;
		
		AST tmp25_AST = null;
		if (inputState.guessing==0) {
			tmp25_AST = (AST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp25_AST);
		}
		match(IDENT);
		{
		_loop40:
		do {
			if ((LA(1)==DOT)) {
				AST tmp26_AST = null;
				if (inputState.guessing==0) {
					tmp26_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp26_AST);
				}
				match(DOT);
				AST tmp27_AST = null;
				if (inputState.guessing==0) {
					tmp27_AST = (AST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp27_AST);
				}
				match(IDENT);
			}
			else {
				break _loop40;
			}
			
		} while (true);
		}
		identifier_AST = (AST)currentAST.root;
		returnAST = identifier_AST;
	}
	
	public final void identifierStar() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST identifierStar_AST = null;
		
		AST tmp28_AST = null;
		if (inputState.guessing==0) {
			tmp28_AST = (AST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp28_AST);
		}
		match(IDENT);
		{
		_loop43:
		do {
			if ((LA(1)==DOT) && (LA(2)==IDENT)) {
				AST tmp29_AST = null;
				if (inputState.guessing==0) {
					tmp29_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp29_AST);
				}
				match(DOT);
				AST tmp30_AST = null;
				if (inputState.guessing==0) {
					tmp30_AST = (AST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp30_AST);
				}
				match(IDENT);
			}
			else {
				break _loop43;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case DOT:
		{
			AST tmp31_AST = null;
			if (inputState.guessing==0) {
				tmp31_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp31_AST);
			}
			match(DOT);
			AST tmp32_AST = null;
			if (inputState.guessing==0) {
				tmp32_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp32_AST);
			}
			match(STAR);
			break;
		}
		case SEMI:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		identifierStar_AST = (AST)currentAST.root;
		returnAST = identifierStar_AST;
	}
	
	public final void modifiers() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST modifiers_AST = null;
		
		{
		_loop47:
		do {
			if ((_tokenSet_8.member(LA(1)))) {
				modifier();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop47;
			}
			
		} while (true);
		}
		if ( inputState.guessing==0 ) {
			modifiers_AST = (AST)currentAST.root;
			modifiers_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(MODIFIERS,"MODIFIERS")).add(modifiers_AST));
			currentAST.root = modifiers_AST;
			currentAST.child = modifiers_AST!=null &&modifiers_AST.getFirstChild()!=null ?
				modifiers_AST.getFirstChild() : modifiers_AST;
			currentAST.advanceChildToEnd();
		}
		modifiers_AST = (AST)currentAST.root;
		returnAST = modifiers_AST;
	}
	
	public final void classDefinition(
		AST modifiers
	) throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST classDefinition_AST = null;
		AST sc_AST = null;
		AST ic_AST = null;
		AST cb_AST = null;
		
		AST tmp33_AST = null;
		if (inputState.guessing==0) {
			tmp33_AST = (AST)astFactory.create(LT(1));
		}
		match(LITERAL_class);
		AST tmp34_AST = null;
		if (inputState.guessing==0) {
			tmp34_AST = (AST)astFactory.create(LT(1));
		}
		match(IDENT);
		superClassClause();
		if (inputState.guessing==0) {
			sc_AST = (AST)returnAST;
		}
		implementsClause();
		if (inputState.guessing==0) {
			ic_AST = (AST)returnAST;
		}
		classBlock();
		if (inputState.guessing==0) {
			cb_AST = (AST)returnAST;
		}
		if ( inputState.guessing==0 ) {
			classDefinition_AST = (AST)currentAST.root;
			classDefinition_AST = (AST)astFactory.make( (new ASTArray(6)).add((AST)astFactory.create(CLASS_DEF,"CLASS_DEF")).add(modifiers).add(tmp34_AST).add(sc_AST).add(ic_AST).add(cb_AST));
			currentAST.root = classDefinition_AST;
			currentAST.child = classDefinition_AST!=null &&classDefinition_AST.getFirstChild()!=null ?
				classDefinition_AST.getFirstChild() : classDefinition_AST;
			currentAST.advanceChildToEnd();
		}
		returnAST = classDefinition_AST;
	}
	
	public final void interfaceDefinition(
		AST modifiers
	) throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST interfaceDefinition_AST = null;
		AST ie_AST = null;
		AST cb_AST = null;
		
		AST tmp35_AST = null;
		if (inputState.guessing==0) {
			tmp35_AST = (AST)astFactory.create(LT(1));
		}
		match(LITERAL_interface);
		AST tmp36_AST = null;
		if (inputState.guessing==0) {
			tmp36_AST = (AST)astFactory.create(LT(1));
		}
		match(IDENT);
		interfaceExtends();
		if (inputState.guessing==0) {
			ie_AST = (AST)returnAST;
		}
		classBlock();
		if (inputState.guessing==0) {
			cb_AST = (AST)returnAST;
		}
		if ( inputState.guessing==0 ) {
			interfaceDefinition_AST = (AST)currentAST.root;
			interfaceDefinition_AST = (AST)astFactory.make( (new ASTArray(5)).add((AST)astFactory.create(INTERFACE_DEF,"INTERFACE_DEF")).add(modifiers).add(tmp36_AST).add(ie_AST).add(cb_AST));
			currentAST.root = interfaceDefinition_AST;
			currentAST.child = interfaceDefinition_AST!=null &&interfaceDefinition_AST.getFirstChild()!=null ?
				interfaceDefinition_AST.getFirstChild() : interfaceDefinition_AST;
			currentAST.advanceChildToEnd();
		}
		returnAST = interfaceDefinition_AST;
	}
	
/** A declaration is the creation of a reference or primitive-type variable
 *  Create a separate Type/Var tree for each var in the var list.
 */
	public final void declaration() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST declaration_AST = null;
		AST m_AST = null;
		AST t_AST = null;
		AST v_AST = null;
		
		modifiers();
		if (inputState.guessing==0) {
			m_AST = (AST)returnAST;
		}
		typeSpec(false);
		if (inputState.guessing==0) {
			t_AST = (AST)returnAST;
		}
		variableDefinitions(m_AST,t_AST);
		if (inputState.guessing==0) {
			v_AST = (AST)returnAST;
		}
		if ( inputState.guessing==0 ) {
			declaration_AST = (AST)currentAST.root;
			declaration_AST = v_AST;
			currentAST.root = declaration_AST;
			currentAST.child = declaration_AST!=null &&declaration_AST.getFirstChild()!=null ?
				declaration_AST.getFirstChild() : declaration_AST;
			currentAST.advanceChildToEnd();
		}
		returnAST = declaration_AST;
	}
	
	public final void typeSpec(
		boolean addImagNode
	) throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST typeSpec_AST = null;
		
		switch ( LA(1)) {
		case IDENT:
		{
			classTypeSpec(addImagNode);
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			typeSpec_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		{
			builtInTypeSpec(addImagNode);
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			typeSpec_AST = (AST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = typeSpec_AST;
	}
	
	public final void variableDefinitions(
		AST mods, AST t
	) throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST variableDefinitions_AST = null;
		
		variableDeclarator(getASTFactory().dupTree(mods),
						   getASTFactory().dupTree(t));
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop79:
		do {
			if ((LA(1)==COMMA)) {
				AST tmp37_AST = null;
				tmp37_AST = (AST)astFactory.create(LT(1));
				match(COMMA);
				variableDeclarator(getASTFactory().dupTree(mods),
							   getASTFactory().dupTree(t));
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop79;
			}
			
		} while (true);
		}
		variableDefinitions_AST = (AST)currentAST.root;
		returnAST = variableDefinitions_AST;
	}
	
	public final void classTypeSpec(
		boolean addImagNode
	) throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST classTypeSpec_AST = null;
		Token  lb = null;
		AST lb_AST = null;
		
		identifier();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop32:
		do {
			if ((LA(1)==LBRACK)) {
				lb = LT(1);
				if (inputState.guessing==0) {
					lb_AST = (AST)astFactory.create(lb);
					astFactory.makeASTRoot(currentAST, lb_AST);
				}
				match(LBRACK);
				if ( inputState.guessing==0 ) {
					lb_AST.setType(ARRAY_DECLARATOR);
				}
				AST tmp38_AST = null;
				tmp38_AST = (AST)astFactory.create(LT(1));
				match(RBRACK);
			}
			else {
				break _loop32;
			}
			
		} while (true);
		}
		if ( inputState.guessing==0 ) {
			classTypeSpec_AST = (AST)currentAST.root;
			
						if ( addImagNode ) {
							classTypeSpec_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(TYPE,"TYPE")).add(classTypeSpec_AST));
						}
					
			currentAST.root = classTypeSpec_AST;
			currentAST.child = classTypeSpec_AST!=null &&classTypeSpec_AST.getFirstChild()!=null ?
				classTypeSpec_AST.getFirstChild() : classTypeSpec_AST;
			currentAST.advanceChildToEnd();
		}
		classTypeSpec_AST = (AST)currentAST.root;
		returnAST = classTypeSpec_AST;
	}
	
	public final void builtInTypeSpec(
		boolean addImagNode
	) throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST builtInTypeSpec_AST = null;
		Token  lb = null;
		AST lb_AST = null;
		
		builtInType();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop35:
		do {
			if ((LA(1)==LBRACK)) {
				lb = LT(1);
				if (inputState.guessing==0) {
					lb_AST = (AST)astFactory.create(lb);
					astFactory.makeASTRoot(currentAST, lb_AST);
				}
				match(LBRACK);
				if ( inputState.guessing==0 ) {
					lb_AST.setType(ARRAY_DECLARATOR);
				}
				AST tmp39_AST = null;
				tmp39_AST = (AST)astFactory.create(LT(1));
				match(RBRACK);
			}
			else {
				break _loop35;
			}
			
		} while (true);
		}
		if ( inputState.guessing==0 ) {
			builtInTypeSpec_AST = (AST)currentAST.root;
			
						if ( addImagNode ) {
							builtInTypeSpec_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(TYPE,"TYPE")).add(builtInTypeSpec_AST));
						}
					
			currentAST.root = builtInTypeSpec_AST;
			currentAST.child = builtInTypeSpec_AST!=null &&builtInTypeSpec_AST.getFirstChild()!=null ?
				builtInTypeSpec_AST.getFirstChild() : builtInTypeSpec_AST;
			currentAST.advanceChildToEnd();
		}
		builtInTypeSpec_AST = (AST)currentAST.root;
		returnAST = builtInTypeSpec_AST;
	}
	
	public final void builtInType() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST builtInType_AST = null;
		
		switch ( LA(1)) {
		case LITERAL_void:
		{
			AST tmp40_AST = null;
			if (inputState.guessing==0) {
				tmp40_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp40_AST);
			}
			match(LITERAL_void);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_boolean:
		{
			AST tmp41_AST = null;
			if (inputState.guessing==0) {
				tmp41_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp41_AST);
			}
			match(LITERAL_boolean);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_byte:
		{
			AST tmp42_AST = null;
			if (inputState.guessing==0) {
				tmp42_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp42_AST);
			}
			match(LITERAL_byte);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_char:
		{
			AST tmp43_AST = null;
			if (inputState.guessing==0) {
				tmp43_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp43_AST);
			}
			match(LITERAL_char);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_short:
		{
			AST tmp44_AST = null;
			if (inputState.guessing==0) {
				tmp44_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp44_AST);
			}
			match(LITERAL_short);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_int:
		{
			AST tmp45_AST = null;
			if (inputState.guessing==0) {
				tmp45_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp45_AST);
			}
			match(LITERAL_int);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_float:
		{
			AST tmp46_AST = null;
			if (inputState.guessing==0) {
				tmp46_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp46_AST);
			}
			match(LITERAL_float);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_long:
		{
			AST tmp47_AST = null;
			if (inputState.guessing==0) {
				tmp47_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp47_AST);
			}
			match(LITERAL_long);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_double:
		{
			AST tmp48_AST = null;
			if (inputState.guessing==0) {
				tmp48_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp48_AST);
			}
			match(LITERAL_double);
			builtInType_AST = (AST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = builtInType_AST;
	}
	
	public final void type() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST type_AST = null;
		
		switch ( LA(1)) {
		case IDENT:
		{
			identifier();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			type_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		{
			builtInType();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			type_AST = (AST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = type_AST;
	}
	
	public final void modifier() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST modifier_AST = null;
		
		switch ( LA(1)) {
		case LITERAL_private:
		{
			AST tmp49_AST = null;
			if (inputState.guessing==0) {
				tmp49_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp49_AST);
			}
			match(LITERAL_private);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_public:
		{
			AST tmp50_AST = null;
			if (inputState.guessing==0) {
				tmp50_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp50_AST);
			}
			match(LITERAL_public);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_protected:
		{
			AST tmp51_AST = null;
			if (inputState.guessing==0) {
				tmp51_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp51_AST);
			}
			match(LITERAL_protected);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_static:
		{
			AST tmp52_AST = null;
			if (inputState.guessing==0) {
				tmp52_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp52_AST);
			}
			match(LITERAL_static);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_transient:
		{
			AST tmp53_AST = null;
			if (inputState.guessing==0) {
				tmp53_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp53_AST);
			}
			match(LITERAL_transient);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case FINAL:
		{
			AST tmp54_AST = null;
			if (inputState.guessing==0) {
				tmp54_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp54_AST);
			}
			match(FINAL);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case ABSTRACT:
		{
			AST tmp55_AST = null;
			if (inputState.guessing==0) {
				tmp55_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp55_AST);
			}
			match(ABSTRACT);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_native:
		{
			AST tmp56_AST = null;
			if (inputState.guessing==0) {
				tmp56_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp56_AST);
			}
			match(LITERAL_native);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_threadsafe:
		{
			AST tmp57_AST = null;
			if (inputState.guessing==0) {
				tmp57_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp57_AST);
			}
			match(LITERAL_threadsafe);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_synchronized:
		{
			AST tmp58_AST = null;
			if (inputState.guessing==0) {
				tmp58_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp58_AST);
			}
			match(LITERAL_synchronized);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_volatile:
		{
			AST tmp59_AST = null;
			if (inputState.guessing==0) {
				tmp59_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp59_AST);
			}
			match(LITERAL_volatile);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		case STRICTFP:
		{
			AST tmp60_AST = null;
			if (inputState.guessing==0) {
				tmp60_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp60_AST);
			}
			match(STRICTFP);
			modifier_AST = (AST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = modifier_AST;
	}
	
	public final void superClassClause() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST superClassClause_AST = null;
		AST id_AST = null;
		
		{
		switch ( LA(1)) {
		case LITERAL_extends:
		{
			AST tmp61_AST = null;
			if (inputState.guessing==0) {
				tmp61_AST = (AST)astFactory.create(LT(1));
			}
			match(LITERAL_extends);
			identifier();
			if (inputState.guessing==0) {
				id_AST = (AST)returnAST;
			}
			break;
		}
		case LCURLY:
		case LITERAL_implements:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			superClassClause_AST = (AST)currentAST.root;
			superClassClause_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(EXTENDS_CLAUSE,"EXTENDS_CLAUSE")).add(id_AST));
			currentAST.root = superClassClause_AST;
			currentAST.child = superClassClause_AST!=null &&superClassClause_AST.getFirstChild()!=null ?
				superClassClause_AST.getFirstChild() : superClassClause_AST;
			currentAST.advanceChildToEnd();
		}
		returnAST = superClassClause_AST;
	}
	
	public final void implementsClause() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST implementsClause_AST = null;
		Token  i = null;
		AST i_AST = null;
		
		{
		switch ( LA(1)) {
		case LITERAL_implements:
		{
			i = LT(1);
			if (inputState.guessing==0) {
				i_AST = (AST)astFactory.create(i);
			}
			match(LITERAL_implements);
			identifier();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			{
			_loop63:
			do {
				if ((LA(1)==COMMA)) {
					AST tmp62_AST = null;
					tmp62_AST = (AST)astFactory.create(LT(1));
					match(COMMA);
					identifier();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else {
					break _loop63;
				}
				
			} while (true);
			}
			break;
		}
		case LCURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			implementsClause_AST = (AST)currentAST.root;
			implementsClause_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(IMPLEMENTS_CLAUSE,"IMPLEMENTS_CLAUSE")).add(implementsClause_AST));
			currentAST.root = implementsClause_AST;
			currentAST.child = implementsClause_AST!=null &&implementsClause_AST.getFirstChild()!=null ?
				implementsClause_AST.getFirstChild() : implementsClause_AST;
			currentAST.advanceChildToEnd();
		}
		implementsClause_AST = (AST)currentAST.root;
		returnAST = implementsClause_AST;
	}
	
	public final void classBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST classBlock_AST = null;
		
		AST tmp63_AST = null;
		tmp63_AST = (AST)astFactory.create(LT(1));
		match(LCURLY);
		{
		_loop55:
		do {
			switch ( LA(1)) {
			case FINAL:
			case ABSTRACT:
			case STRICTFP:
			case IDENT:
			case LCURLY:
			case LITERAL_void:
			case LITERAL_boolean:
			case LITERAL_byte:
			case LITERAL_char:
			case LITERAL_short:
			case LITERAL_int:
			case LITERAL_float:
			case LITERAL_long:
			case LITERAL_double:
			case LITERAL_private:
			case LITERAL_public:
			case LITERAL_protected:
			case LITERAL_static:
			case LITERAL_transient:
			case LITERAL_native:
			case LITERAL_threadsafe:
			case LITERAL_synchronized:
			case LITERAL_volatile:
			case LITERAL_class:
			case LITERAL_interface:
			{
				field();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				break;
			}
			case SEMI:
			{
				AST tmp64_AST = null;
				tmp64_AST = (AST)astFactory.create(LT(1));
				match(SEMI);
				break;
			}
			default:
			{
				break _loop55;
			}
			}
		} while (true);
		}
		AST tmp65_AST = null;
		tmp65_AST = (AST)astFactory.create(LT(1));
		match(RCURLY);
		if ( inputState.guessing==0 ) {
			classBlock_AST = (AST)currentAST.root;
			classBlock_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(OBJBLOCK,"OBJBLOCK")).add(classBlock_AST));
			currentAST.root = classBlock_AST;
			currentAST.child = classBlock_AST!=null &&classBlock_AST.getFirstChild()!=null ?
				classBlock_AST.getFirstChild() : classBlock_AST;
			currentAST.advanceChildToEnd();
		}
		classBlock_AST = (AST)currentAST.root;
		returnAST = classBlock_AST;
	}
	
	public final void interfaceExtends() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST interfaceExtends_AST = null;
		Token  e = null;
		AST e_AST = null;
		
		{
		switch ( LA(1)) {
		case LITERAL_extends:
		{
			e = LT(1);
			if (inputState.guessing==0) {
				e_AST = (AST)astFactory.create(e);
			}
			match(LITERAL_extends);
			identifier();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			{
			_loop59:
			do {
				if ((LA(1)==COMMA)) {
					AST tmp66_AST = null;
					tmp66_AST = (AST)astFactory.create(LT(1));
					match(COMMA);
					identifier();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else {
					break _loop59;
				}
				
			} while (true);
			}
			break;
		}
		case LCURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			interfaceExtends_AST = (AST)currentAST.root;
			interfaceExtends_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(EXTENDS_CLAUSE,"EXTENDS_CLAUSE")).add(interfaceExtends_AST));
			currentAST.root = interfaceExtends_AST;
			currentAST.child = interfaceExtends_AST!=null &&interfaceExtends_AST.getFirstChild()!=null ?
				interfaceExtends_AST.getFirstChild() : interfaceExtends_AST;
			currentAST.advanceChildToEnd();
		}
		interfaceExtends_AST = (AST)currentAST.root;
		returnAST = interfaceExtends_AST;
	}
	
	public final void field() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST field_AST = null;
		AST mods_AST = null;
		AST h_AST = null;
		AST s_AST = null;
		AST cd_AST = null;
		AST id_AST = null;
		AST t_AST = null;
		AST param_AST = null;
		AST rt_AST = null;
		AST tc_AST = null;
		AST s2_AST = null;
		AST v_AST = null;
		AST s3_AST = null;
		AST s4_AST = null;
		
		if ((_tokenSet_9.member(LA(1))) && (_tokenSet_10.member(LA(2)))) {
			modifiers();
			if (inputState.guessing==0) {
				mods_AST = (AST)returnAST;
			}
			{
			switch ( LA(1)) {
			case LITERAL_class:
			{
				classDefinition(mods_AST);
				if (inputState.guessing==0) {
					cd_AST = (AST)returnAST;
				}
				if ( inputState.guessing==0 ) {
					field_AST = (AST)currentAST.root;
					field_AST = cd_AST;
					currentAST.root = field_AST;
					currentAST.child = field_AST!=null &&field_AST.getFirstChild()!=null ?
						field_AST.getFirstChild() : field_AST;
					currentAST.advanceChildToEnd();
				}
				break;
			}
			case LITERAL_interface:
			{
				interfaceDefinition(mods_AST);
				if (inputState.guessing==0) {
					id_AST = (AST)returnAST;
				}
				if ( inputState.guessing==0 ) {
					field_AST = (AST)currentAST.root;
					field_AST = id_AST;
					currentAST.root = field_AST;
					currentAST.child = field_AST!=null &&field_AST.getFirstChild()!=null ?
						field_AST.getFirstChild() : field_AST;
					currentAST.advanceChildToEnd();
				}
				break;
			}
			default:
				if ((LA(1)==IDENT) && (LA(2)==LPAREN)) {
					ctorHead();
					if (inputState.guessing==0) {
						h_AST = (AST)returnAST;
					}
					constructorBody();
					if (inputState.guessing==0) {
						s_AST = (AST)returnAST;
					}
					if ( inputState.guessing==0 ) {
						field_AST = (AST)currentAST.root;
						field_AST = (AST)astFactory.make( (new ASTArray(4)).add((AST)astFactory.create(CTOR_DEF,"CTOR_DEF")).add(mods_AST).add(h_AST).add(s_AST));
						currentAST.root = field_AST;
						currentAST.child = field_AST!=null &&field_AST.getFirstChild()!=null ?
							field_AST.getFirstChild() : field_AST;
						currentAST.advanceChildToEnd();
					}
				}
				else if ((_tokenSet_11.member(LA(1))) && (_tokenSet_12.member(LA(2)))) {
					typeSpec(false);
					if (inputState.guessing==0) {
						t_AST = (AST)returnAST;
					}
					{
					if ((LA(1)==IDENT) && (LA(2)==LPAREN)) {
						AST tmp67_AST = null;
						if (inputState.guessing==0) {
							tmp67_AST = (AST)astFactory.create(LT(1));
						}
						match(IDENT);
						AST tmp68_AST = null;
						tmp68_AST = (AST)astFactory.create(LT(1));
						match(LPAREN);
						parameterDeclarationList();
						if (inputState.guessing==0) {
							param_AST = (AST)returnAST;
						}
						AST tmp69_AST = null;
						tmp69_AST = (AST)astFactory.create(LT(1));
						match(RPAREN);
						declaratorBrackets(t_AST);
						if (inputState.guessing==0) {
							rt_AST = (AST)returnAST;
						}
						{
						switch ( LA(1)) {
						case LITERAL_throws:
						{
							throwsClause();
							if (inputState.guessing==0) {
								tc_AST = (AST)returnAST;
							}
							break;
						}
						case LCURLY:
						case SEMI:
						{
							break;
						}
						default:
						{
							throw new NoViableAltException(LT(1), getFilename());
						}
						}
						}
						{
						switch ( LA(1)) {
						case LCURLY:
						{
							compoundStatement();
							if (inputState.guessing==0) {
								s2_AST = (AST)returnAST;
							}
							break;
						}
						case SEMI:
						{
							AST tmp70_AST = null;
							if (inputState.guessing==0) {
								tmp70_AST = (AST)astFactory.create(LT(1));
							}
							match(SEMI);
							break;
						}
						default:
						{
							throw new NoViableAltException(LT(1), getFilename());
						}
						}
						}
						if ( inputState.guessing==0 ) {
							field_AST = (AST)currentAST.root;
							field_AST = (AST)astFactory.make( (new ASTArray(7)).add((AST)astFactory.create(METHOD_DEF,"METHOD_DEF")).add(mods_AST).add((AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(TYPE,"TYPE")).add(rt_AST))).add(tmp67_AST).add(param_AST).add(tc_AST).add(s2_AST));
							currentAST.root = field_AST;
							currentAST.child = field_AST!=null &&field_AST.getFirstChild()!=null ?
								field_AST.getFirstChild() : field_AST;
							currentAST.advanceChildToEnd();
						}
					}
					else if ((LA(1)==IDENT) && (_tokenSet_13.member(LA(2)))) {
						variableDefinitions(mods_AST,t_AST);
						if (inputState.guessing==0) {
							v_AST = (AST)returnAST;
						}
						AST tmp71_AST = null;
						if (inputState.guessing==0) {
							tmp71_AST = (AST)astFactory.create(LT(1));
						}
						match(SEMI);
						if ( inputState.guessing==0 ) {
							field_AST = (AST)currentAST.root;
							field_AST = v_AST;
							currentAST.root = field_AST;
							currentAST.child = field_AST!=null &&field_AST.getFirstChild()!=null ?
								field_AST.getFirstChild() : field_AST;
							currentAST.advanceChildToEnd();
						}
					}
					else {
						throw new NoViableAltException(LT(1), getFilename());
					}
					
					}
				}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		else if ((LA(1)==LITERAL_static) && (LA(2)==LCURLY)) {
			AST tmp72_AST = null;
			if (inputState.guessing==0) {
				tmp72_AST = (AST)astFactory.create(LT(1));
			}
			match(LITERAL_static);
			compoundStatement();
			if (inputState.guessing==0) {
				s3_AST = (AST)returnAST;
			}
			if ( inputState.guessing==0 ) {
				field_AST = (AST)currentAST.root;
				field_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(STATIC_INIT,"STATIC_INIT")).add(s3_AST));
				currentAST.root = field_AST;
				currentAST.child = field_AST!=null &&field_AST.getFirstChild()!=null ?
					field_AST.getFirstChild() : field_AST;
				currentAST.advanceChildToEnd();
			}
		}
		else if ((LA(1)==LCURLY)) {
			compoundStatement();
			if (inputState.guessing==0) {
				s4_AST = (AST)returnAST;
			}
			if ( inputState.guessing==0 ) {
				field_AST = (AST)currentAST.root;
				field_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(INSTANCE_INIT,"INSTANCE_INIT")).add(s4_AST));
				currentAST.root = field_AST;
				currentAST.child = field_AST!=null &&field_AST.getFirstChild()!=null ?
					field_AST.getFirstChild() : field_AST;
				currentAST.advanceChildToEnd();
			}
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		returnAST = field_AST;
	}
	
	public final void ctorHead() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST ctorHead_AST = null;
		
		AST tmp73_AST = null;
		if (inputState.guessing==0) {
			tmp73_AST = (AST)astFactory.create(LT(1));
			astFactory.addASTChild(currentAST, tmp73_AST);
		}
		match(IDENT);
		AST tmp74_AST = null;
		tmp74_AST = (AST)astFactory.create(LT(1));
		match(LPAREN);
		parameterDeclarationList();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		AST tmp75_AST = null;
		tmp75_AST = (AST)astFactory.create(LT(1));
		match(RPAREN);
		{
		switch ( LA(1)) {
		case LITERAL_throws:
		{
			throwsClause();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case LCURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		ctorHead_AST = (AST)currentAST.root;
		returnAST = ctorHead_AST;
	}
	
	public final void constructorBody() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST constructorBody_AST = null;
		Token  lc = null;
		AST lc_AST = null;
		
		lc = LT(1);
		if (inputState.guessing==0) {
			lc_AST = (AST)astFactory.create(lc);
			astFactory.makeASTRoot(currentAST, lc_AST);
		}
		match(LCURLY);
		if ( inputState.guessing==0 ) {
			lc_AST.setType(SLIST);
		}
		{
		boolean synPredMatched72 = false;
		if (((_tokenSet_14.member(LA(1))) && (_tokenSet_15.member(LA(2))))) {
			int _m72 = mark();
			synPredMatched72 = true;
			inputState.guessing++;
			try {
				{
				explicitConstructorInvocation();
				}
			}
			catch (RecognitionException pe) {
				synPredMatched72 = false;
			}
			rewind(_m72);
			inputState.guessing--;
		}
		if ( synPredMatched72 ) {
			explicitConstructorInvocation();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
		}
		else if ((_tokenSet_16.member(LA(1))) && (_tokenSet_17.member(LA(2)))) {
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		}
		{
		_loop74:
		do {
			if ((_tokenSet_6.member(LA(1)))) {
				statement();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop74;
			}
			
		} while (true);
		}
		AST tmp76_AST = null;
		tmp76_AST = (AST)astFactory.create(LT(1));
		match(RCURLY);
		constructorBody_AST = (AST)currentAST.root;
		returnAST = constructorBody_AST;
	}
	
	public final void declaratorBrackets(
		AST typ
	) throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST declaratorBrackets_AST = null;
		Token  lb = null;
		AST lb_AST = null;
		
		if ( inputState.guessing==0 ) {
			declaratorBrackets_AST = (AST)currentAST.root;
			declaratorBrackets_AST=typ;
			currentAST.root = declaratorBrackets_AST;
			currentAST.child = declaratorBrackets_AST!=null &&declaratorBrackets_AST.getFirstChild()!=null ?
				declaratorBrackets_AST.getFirstChild() : declaratorBrackets_AST;
			currentAST.advanceChildToEnd();
		}
		{
		_loop83:
		do {
			if ((LA(1)==LBRACK)) {
				lb = LT(1);
				if (inputState.guessing==0) {
					lb_AST = (AST)astFactory.create(lb);
					astFactory.makeASTRoot(currentAST, lb_AST);
				}
				match(LBRACK);
				if ( inputState.guessing==0 ) {
					lb_AST.setType(ARRAY_DECLARATOR);
				}
				AST tmp77_AST = null;
				tmp77_AST = (AST)astFactory.create(LT(1));
				match(RBRACK);
			}
			else {
				break _loop83;
			}
			
		} while (true);
		}
		declaratorBrackets_AST = (AST)currentAST.root;
		returnAST = declaratorBrackets_AST;
	}
	
	public final void throwsClause() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST throwsClause_AST = null;
		
		AST tmp78_AST = null;
		if (inputState.guessing==0) {
			tmp78_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp78_AST);
		}
		match(LITERAL_throws);
		identifier();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop96:
		do {
			if ((LA(1)==COMMA)) {
				AST tmp79_AST = null;
				tmp79_AST = (AST)astFactory.create(LT(1));
				match(COMMA);
				identifier();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop96;
			}
			
		} while (true);
		}
		throwsClause_AST = (AST)currentAST.root;
		returnAST = throwsClause_AST;
	}
	
	public final void explicitConstructorInvocation() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST explicitConstructorInvocation_AST = null;
		Token  lp1 = null;
		AST lp1_AST = null;
		Token  lp2 = null;
		AST lp2_AST = null;
		Token  lp3 = null;
		AST lp3_AST = null;
		
		{
		if ((LA(1)==LITERAL_this) && (LA(2)==LPAREN)) {
			AST tmp80_AST = null;
			tmp80_AST = (AST)astFactory.create(LT(1));
			match(LITERAL_this);
			lp1 = LT(1);
			if (inputState.guessing==0) {
				lp1_AST = (AST)astFactory.create(lp1);
				astFactory.makeASTRoot(currentAST, lp1_AST);
			}
			match(LPAREN);
			argList();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp81_AST = null;
			tmp81_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			AST tmp82_AST = null;
			tmp82_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			if ( inputState.guessing==0 ) {
				lp1_AST.setType(CTOR_CALL);
			}
		}
		else if ((LA(1)==LITERAL_super) && (LA(2)==LPAREN)) {
			AST tmp83_AST = null;
			tmp83_AST = (AST)astFactory.create(LT(1));
			match(LITERAL_super);
			lp2 = LT(1);
			if (inputState.guessing==0) {
				lp2_AST = (AST)astFactory.create(lp2);
				astFactory.makeASTRoot(currentAST, lp2_AST);
			}
			match(LPAREN);
			argList();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp84_AST = null;
			tmp84_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			AST tmp85_AST = null;
			tmp85_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			if ( inputState.guessing==0 ) {
				lp2_AST.setType(SUPER_CTOR_CALL);
			}
		}
		else if ((_tokenSet_14.member(LA(1))) && (_tokenSet_15.member(LA(2)))) {
			primaryExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp86_AST = null;
			tmp86_AST = (AST)astFactory.create(LT(1));
			match(DOT);
			AST tmp87_AST = null;
			tmp87_AST = (AST)astFactory.create(LT(1));
			match(LITERAL_super);
			lp3 = LT(1);
			if (inputState.guessing==0) {
				lp3_AST = (AST)astFactory.create(lp3);
				astFactory.makeASTRoot(currentAST, lp3_AST);
			}
			match(LPAREN);
			argList();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp88_AST = null;
			tmp88_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			AST tmp89_AST = null;
			tmp89_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			if ( inputState.guessing==0 ) {
				lp3_AST.setType(SUPER_CTOR_CALL);
			}
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		}
		explicitConstructorInvocation_AST = (AST)currentAST.root;
		returnAST = explicitConstructorInvocation_AST;
	}
	
	public final void statement() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST statement_AST = null;
		AST m_AST = null;
		Token  c = null;
		AST c_AST = null;
		Token  s = null;
		AST s_AST = null;
		
		switch ( LA(1)) {
		case LCURLY:
		{
			compoundStatement();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_if:
		{
			AST tmp90_AST = null;
			if (inputState.guessing==0) {
				tmp90_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp90_AST);
			}
			match(LITERAL_if);
			AST tmp91_AST = null;
			tmp91_AST = (AST)astFactory.create(LT(1));
			match(LPAREN);
			expression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp92_AST = null;
			tmp92_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			statement();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			{
			if ((LA(1)==LITERAL_else) && (_tokenSet_6.member(LA(2)))) {
				AST tmp93_AST = null;
				tmp93_AST = (AST)astFactory.create(LT(1));
				match(LITERAL_else);
				statement();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else if ((_tokenSet_18.member(LA(1))) && (_tokenSet_19.member(LA(2)))) {
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_for:
		{
			AST tmp94_AST = null;
			if (inputState.guessing==0) {
				tmp94_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp94_AST);
			}
			match(LITERAL_for);
			AST tmp95_AST = null;
			tmp95_AST = (AST)astFactory.create(LT(1));
			match(LPAREN);
			forInit();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp96_AST = null;
			tmp96_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			forCond();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp97_AST = null;
			tmp97_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			forIter();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp98_AST = null;
			tmp98_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			statement();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_while:
		{
			AST tmp99_AST = null;
			if (inputState.guessing==0) {
				tmp99_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp99_AST);
			}
			match(LITERAL_while);
			AST tmp100_AST = null;
			tmp100_AST = (AST)astFactory.create(LT(1));
			match(LPAREN);
			expression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp101_AST = null;
			tmp101_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			statement();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_do:
		{
			AST tmp102_AST = null;
			if (inputState.guessing==0) {
				tmp102_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp102_AST);
			}
			match(LITERAL_do);
			statement();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp103_AST = null;
			tmp103_AST = (AST)astFactory.create(LT(1));
			match(LITERAL_while);
			AST tmp104_AST = null;
			tmp104_AST = (AST)astFactory.create(LT(1));
			match(LPAREN);
			expression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp105_AST = null;
			tmp105_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			AST tmp106_AST = null;
			tmp106_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_break:
		{
			AST tmp107_AST = null;
			if (inputState.guessing==0) {
				tmp107_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp107_AST);
			}
			match(LITERAL_break);
			{
			switch ( LA(1)) {
			case IDENT:
			{
				AST tmp108_AST = null;
				if (inputState.guessing==0) {
					tmp108_AST = (AST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp108_AST);
				}
				match(IDENT);
				break;
			}
			case SEMI:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			AST tmp109_AST = null;
			tmp109_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_continue:
		{
			AST tmp110_AST = null;
			if (inputState.guessing==0) {
				tmp110_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp110_AST);
			}
			match(LITERAL_continue);
			{
			switch ( LA(1)) {
			case IDENT:
			{
				AST tmp111_AST = null;
				if (inputState.guessing==0) {
					tmp111_AST = (AST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp111_AST);
				}
				match(IDENT);
				break;
			}
			case SEMI:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			AST tmp112_AST = null;
			tmp112_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_return:
		{
			AST tmp113_AST = null;
			if (inputState.guessing==0) {
				tmp113_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp113_AST);
			}
			match(LITERAL_return);
			{
			switch ( LA(1)) {
			case IDENT:
			case LPAREN:
			case LITERAL_void:
			case LITERAL_boolean:
			case LITERAL_byte:
			case LITERAL_char:
			case LITERAL_short:
			case LITERAL_int:
			case LITERAL_float:
			case LITERAL_long:
			case LITERAL_double:
			case LITERAL_this:
			case LITERAL_super:
			case PLUS:
			case MINUS:
			case INC:
			case DEC:
			case BNOT:
			case LNOT:
			case LITERAL_true:
			case LITERAL_false:
			case LITERAL_null:
			case LITERAL_new:
			case NUM_INT:
			case CHAR_LITERAL:
			case STRING_LITERAL:
			case NUM_FLOAT:
			case NUM_LONG:
			case NUM_DOUBLE:
			{
				expression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				break;
			}
			case SEMI:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			AST tmp114_AST = null;
			tmp114_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_switch:
		{
			AST tmp115_AST = null;
			if (inputState.guessing==0) {
				tmp115_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp115_AST);
			}
			match(LITERAL_switch);
			AST tmp116_AST = null;
			tmp116_AST = (AST)astFactory.create(LT(1));
			match(LPAREN);
			expression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp117_AST = null;
			tmp117_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			AST tmp118_AST = null;
			tmp118_AST = (AST)astFactory.create(LT(1));
			match(LCURLY);
			{
			_loop115:
			do {
				if ((LA(1)==LITERAL_case||LA(1)==LITERAL_default)) {
					casesGroup();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else {
					break _loop115;
				}
				
			} while (true);
			}
			AST tmp119_AST = null;
			tmp119_AST = (AST)astFactory.create(LT(1));
			match(RCURLY);
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_try:
		{
			tryBlock();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			statement_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_throw:
		{
			AST tmp120_AST = null;
			if (inputState.guessing==0) {
				tmp120_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp120_AST);
			}
			match(LITERAL_throw);
			expression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp121_AST = null;
			tmp121_AST = (AST)astFactory.create(LT(1));
			match(SEMI);
			statement_AST = (AST)currentAST.root;
			break;
		}
		case SEMI:
		{
			s = LT(1);
			if (inputState.guessing==0) {
				s_AST = (AST)astFactory.create(s);
				astFactory.addASTChild(currentAST, s_AST);
			}
			match(SEMI);
			if ( inputState.guessing==0 ) {
				s_AST.setType(EMPTY_STAT);
			}
			statement_AST = (AST)currentAST.root;
			break;
		}
		default:
			boolean synPredMatched109 = false;
			if (((_tokenSet_20.member(LA(1))) && (_tokenSet_21.member(LA(2))))) {
				int _m109 = mark();
				synPredMatched109 = true;
				inputState.guessing++;
				try {
					{
					declaration();
					}
				}
				catch (RecognitionException pe) {
					synPredMatched109 = false;
				}
				rewind(_m109);
				inputState.guessing--;
			}
			if ( synPredMatched109 ) {
				declaration();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				AST tmp122_AST = null;
				tmp122_AST = (AST)astFactory.create(LT(1));
				match(SEMI);
				statement_AST = (AST)currentAST.root;
			}
			else if ((_tokenSet_4.member(LA(1))) && (_tokenSet_22.member(LA(2)))) {
				expression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				AST tmp123_AST = null;
				tmp123_AST = (AST)astFactory.create(LT(1));
				match(SEMI);
				statement_AST = (AST)currentAST.root;
			}
			else if ((_tokenSet_23.member(LA(1))) && (_tokenSet_24.member(LA(2)))) {
				modifiers();
				if (inputState.guessing==0) {
					m_AST = (AST)returnAST;
				}
				classDefinition(m_AST);
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				statement_AST = (AST)currentAST.root;
			}
			else if ((LA(1)==IDENT) && (LA(2)==COLON)) {
				AST tmp124_AST = null;
				if (inputState.guessing==0) {
					tmp124_AST = (AST)astFactory.create(LT(1));
					astFactory.addASTChild(currentAST, tmp124_AST);
				}
				match(IDENT);
				c = LT(1);
				if (inputState.guessing==0) {
					c_AST = (AST)astFactory.create(c);
					astFactory.makeASTRoot(currentAST, c_AST);
				}
				match(COLON);
				if ( inputState.guessing==0 ) {
					c_AST.setType(LABELED_STAT);
				}
				statement();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				statement_AST = (AST)currentAST.root;
			}
			else if ((LA(1)==LITERAL_synchronized) && (LA(2)==LPAREN)) {
				AST tmp125_AST = null;
				if (inputState.guessing==0) {
					tmp125_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp125_AST);
				}
				match(LITERAL_synchronized);
				AST tmp126_AST = null;
				tmp126_AST = (AST)astFactory.create(LT(1));
				match(LPAREN);
				expression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				AST tmp127_AST = null;
				tmp127_AST = (AST)astFactory.create(LT(1));
				match(RPAREN);
				compoundStatement();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				statement_AST = (AST)currentAST.root;
			}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = statement_AST;
	}
	
	public final void argList() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST argList_AST = null;
		
		{
		switch ( LA(1)) {
		case IDENT:
		case LPAREN:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case LITERAL_this:
		case LITERAL_super:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			expressionList();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case RPAREN:
		{
			if ( inputState.guessing==0 ) {
				argList_AST = (AST)currentAST.root;
				argList_AST = (AST)astFactory.create(ELIST,"ELIST");
				currentAST.root = argList_AST;
				currentAST.child = argList_AST!=null &&argList_AST.getFirstChild()!=null ?
					argList_AST.getFirstChild() : argList_AST;
				currentAST.advanceChildToEnd();
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		argList_AST = (AST)currentAST.root;
		returnAST = argList_AST;
	}
	
	public final void primaryExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST primaryExpression_AST = null;
		Token  lbt = null;
		AST lbt_AST = null;
		
		switch ( LA(1)) {
		case IDENT:
		{
			AST tmp128_AST = null;
			if (inputState.guessing==0) {
				tmp128_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp128_AST);
			}
			match(IDENT);
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			constant();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_true:
		{
			AST tmp129_AST = null;
			if (inputState.guessing==0) {
				tmp129_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp129_AST);
			}
			match(LITERAL_true);
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_false:
		{
			AST tmp130_AST = null;
			if (inputState.guessing==0) {
				tmp130_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp130_AST);
			}
			match(LITERAL_false);
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_this:
		{
			AST tmp131_AST = null;
			if (inputState.guessing==0) {
				tmp131_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp131_AST);
			}
			match(LITERAL_this);
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_null:
		{
			AST tmp132_AST = null;
			if (inputState.guessing==0) {
				tmp132_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp132_AST);
			}
			match(LITERAL_null);
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_new:
		{
			newExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case LPAREN:
		{
			AST tmp133_AST = null;
			tmp133_AST = (AST)astFactory.create(LT(1));
			match(LPAREN);
			assignmentExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp134_AST = null;
			tmp134_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_super:
		{
			AST tmp135_AST = null;
			if (inputState.guessing==0) {
				tmp135_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp135_AST);
			}
			match(LITERAL_super);
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		{
			builtInType();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			{
			_loop196:
			do {
				if ((LA(1)==LBRACK)) {
					lbt = LT(1);
					if (inputState.guessing==0) {
						lbt_AST = (AST)astFactory.create(lbt);
						astFactory.makeASTRoot(currentAST, lbt_AST);
					}
					match(LBRACK);
					if ( inputState.guessing==0 ) {
						lbt_AST.setType(ARRAY_DECLARATOR);
					}
					AST tmp136_AST = null;
					tmp136_AST = (AST)astFactory.create(LT(1));
					match(RBRACK);
				}
				else {
					break _loop196;
				}
				
			} while (true);
			}
			AST tmp137_AST = null;
			if (inputState.guessing==0) {
				tmp137_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp137_AST);
			}
			match(DOT);
			AST tmp138_AST = null;
			if (inputState.guessing==0) {
				tmp138_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp138_AST);
			}
			match(LITERAL_class);
			primaryExpression_AST = (AST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = primaryExpression_AST;
	}
	
/** Declaration of a variable.  This can be a class/instance variable,
 *   or a local variable in a method
 * It can also include possible initialization.
 */
	public final void variableDeclarator(
		AST mods, AST t
	) throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST variableDeclarator_AST = null;
		Token  id = null;
		AST id_AST = null;
		AST d_AST = null;
		AST v_AST = null;
		
		id = LT(1);
		if (inputState.guessing==0) {
			id_AST = (AST)astFactory.create(id);
		}
		match(IDENT);
		declaratorBrackets(t);
		if (inputState.guessing==0) {
			d_AST = (AST)returnAST;
		}
		varInitializer();
		if (inputState.guessing==0) {
			v_AST = (AST)returnAST;
		}
		if ( inputState.guessing==0 ) {
			variableDeclarator_AST = (AST)currentAST.root;
			variableDeclarator_AST = (AST)astFactory.make( (new ASTArray(5)).add((AST)astFactory.create(VARIABLE_DEF,"VARIABLE_DEF")).add(mods).add((AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(TYPE,"TYPE")).add(d_AST))).add(id_AST).add(v_AST));
			currentAST.root = variableDeclarator_AST;
			currentAST.child = variableDeclarator_AST!=null &&variableDeclarator_AST.getFirstChild()!=null ?
				variableDeclarator_AST.getFirstChild() : variableDeclarator_AST;
			currentAST.advanceChildToEnd();
		}
		returnAST = variableDeclarator_AST;
	}
	
	public final void varInitializer() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST varInitializer_AST = null;
		
		{
		switch ( LA(1)) {
		case ASSIGN:
		{
			AST tmp139_AST = null;
			if (inputState.guessing==0) {
				tmp139_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp139_AST);
			}
			match(ASSIGN);
			initializer();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case SEMI:
		case COMMA:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		varInitializer_AST = (AST)currentAST.root;
		returnAST = varInitializer_AST;
	}
	
	public final void initializer() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST initializer_AST = null;
		
		switch ( LA(1)) {
		case IDENT:
		case LPAREN:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case LITERAL_this:
		case LITERAL_super:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			expression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			initializer_AST = (AST)currentAST.root;
			break;
		}
		case LCURLY:
		{
			arrayInitializer();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			initializer_AST = (AST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = initializer_AST;
	}
	
	public final void arrayInitializer() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST arrayInitializer_AST = null;
		Token  lc = null;
		AST lc_AST = null;
		
		lc = LT(1);
		if (inputState.guessing==0) {
			lc_AST = (AST)astFactory.create(lc);
			astFactory.makeASTRoot(currentAST, lc_AST);
		}
		match(LCURLY);
		if ( inputState.guessing==0 ) {
			lc_AST.setType(ARRAY_INIT);
		}
		{
		switch ( LA(1)) {
		case IDENT:
		case LCURLY:
		case LPAREN:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case LITERAL_this:
		case LITERAL_super:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			initializer();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			{
			_loop89:
			do {
				if ((LA(1)==COMMA) && (_tokenSet_25.member(LA(2)))) {
					AST tmp140_AST = null;
					tmp140_AST = (AST)astFactory.create(LT(1));
					match(COMMA);
					initializer();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else {
					break _loop89;
				}
				
			} while (true);
			}
			{
			switch ( LA(1)) {
			case COMMA:
			{
				AST tmp141_AST = null;
				tmp141_AST = (AST)astFactory.create(LT(1));
				match(COMMA);
				break;
			}
			case RCURLY:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		case RCURLY:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		AST tmp142_AST = null;
		tmp142_AST = (AST)astFactory.create(LT(1));
		match(RCURLY);
		arrayInitializer_AST = (AST)currentAST.root;
		returnAST = arrayInitializer_AST;
	}
	
	public final void expression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST expression_AST = null;
		
		assignmentExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		if ( inputState.guessing==0 ) {
			expression_AST = (AST)currentAST.root;
			expression_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(EXPR,"EXPR")).add(expression_AST));
			currentAST.root = expression_AST;
			currentAST.child = expression_AST!=null &&expression_AST.getFirstChild()!=null ?
				expression_AST.getFirstChild() : expression_AST;
			currentAST.advanceChildToEnd();
		}
		expression_AST = (AST)currentAST.root;
		returnAST = expression_AST;
	}
	
	public final void parameterModifier() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST parameterModifier_AST = null;
		Token  f = null;
		AST f_AST = null;
		
		{
		switch ( LA(1)) {
		case FINAL:
		{
			f = LT(1);
			if (inputState.guessing==0) {
				f_AST = (AST)astFactory.create(f);
				astFactory.addASTChild(currentAST, f_AST);
			}
			match(FINAL);
			break;
		}
		case IDENT:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			parameterModifier_AST = (AST)currentAST.root;
			parameterModifier_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(MODIFIERS,"MODIFIERS")).add(f_AST));
			currentAST.root = parameterModifier_AST;
			currentAST.child = parameterModifier_AST!=null &&parameterModifier_AST.getFirstChild()!=null ?
				parameterModifier_AST.getFirstChild() : parameterModifier_AST;
			currentAST.advanceChildToEnd();
		}
		parameterModifier_AST = (AST)currentAST.root;
		returnAST = parameterModifier_AST;
	}
	
	public final void forInit() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST forInit_AST = null;
		
		{
		boolean synPredMatched127 = false;
		if (((_tokenSet_20.member(LA(1))) && (_tokenSet_21.member(LA(2))))) {
			int _m127 = mark();
			synPredMatched127 = true;
			inputState.guessing++;
			try {
				{
				declaration();
				}
			}
			catch (RecognitionException pe) {
				synPredMatched127 = false;
			}
			rewind(_m127);
			inputState.guessing--;
		}
		if ( synPredMatched127 ) {
			declaration();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
		}
		else if ((_tokenSet_4.member(LA(1))) && (_tokenSet_26.member(LA(2)))) {
			expressionList();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
		}
		else if ((LA(1)==SEMI)) {
		}
		else {
			throw new NoViableAltException(LT(1), getFilename());
		}
		
		}
		if ( inputState.guessing==0 ) {
			forInit_AST = (AST)currentAST.root;
			forInit_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(FOR_INIT,"FOR_INIT")).add(forInit_AST));
			currentAST.root = forInit_AST;
			currentAST.child = forInit_AST!=null &&forInit_AST.getFirstChild()!=null ?
				forInit_AST.getFirstChild() : forInit_AST;
			currentAST.advanceChildToEnd();
		}
		forInit_AST = (AST)currentAST.root;
		returnAST = forInit_AST;
	}
	
	public final void forCond() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST forCond_AST = null;
		
		{
		switch ( LA(1)) {
		case IDENT:
		case LPAREN:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case LITERAL_this:
		case LITERAL_super:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			expression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case SEMI:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			forCond_AST = (AST)currentAST.root;
			forCond_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(FOR_CONDITION,"FOR_CONDITION")).add(forCond_AST));
			currentAST.root = forCond_AST;
			currentAST.child = forCond_AST!=null &&forCond_AST.getFirstChild()!=null ?
				forCond_AST.getFirstChild() : forCond_AST;
			currentAST.advanceChildToEnd();
		}
		forCond_AST = (AST)currentAST.root;
		returnAST = forCond_AST;
	}
	
	public final void forIter() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST forIter_AST = null;
		
		{
		switch ( LA(1)) {
		case IDENT:
		case LPAREN:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case LITERAL_this:
		case LITERAL_super:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			expressionList();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case RPAREN:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		if ( inputState.guessing==0 ) {
			forIter_AST = (AST)currentAST.root;
			forIter_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(FOR_ITERATOR,"FOR_ITERATOR")).add(forIter_AST));
			currentAST.root = forIter_AST;
			currentAST.child = forIter_AST!=null &&forIter_AST.getFirstChild()!=null ?
				forIter_AST.getFirstChild() : forIter_AST;
			currentAST.advanceChildToEnd();
		}
		forIter_AST = (AST)currentAST.root;
		returnAST = forIter_AST;
	}
	
	public final void casesGroup() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST casesGroup_AST = null;
		
		{
		int _cnt118=0;
		_loop118:
		do {
			if ((LA(1)==LITERAL_case||LA(1)==LITERAL_default) && (_tokenSet_27.member(LA(2)))) {
				aCase();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				if ( _cnt118>=1 ) { break _loop118; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt118++;
		} while (true);
		}
		caseSList();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		if ( inputState.guessing==0 ) {
			casesGroup_AST = (AST)currentAST.root;
			casesGroup_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(CASE_GROUP,"CASE_GROUP")).add(casesGroup_AST));
			currentAST.root = casesGroup_AST;
			currentAST.child = casesGroup_AST!=null &&casesGroup_AST.getFirstChild()!=null ?
				casesGroup_AST.getFirstChild() : casesGroup_AST;
			currentAST.advanceChildToEnd();
		}
		casesGroup_AST = (AST)currentAST.root;
		returnAST = casesGroup_AST;
	}
	
	public final void tryBlock() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST tryBlock_AST = null;
		
		AST tmp143_AST = null;
		if (inputState.guessing==0) {
			tmp143_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp143_AST);
		}
		match(LITERAL_try);
		compoundStatement();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop134:
		do {
			if ((LA(1)==LITERAL_catch)) {
				handler();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop134;
			}
			
		} while (true);
		}
		{
		switch ( LA(1)) {
		case LITERAL_finally:
		{
			AST tmp144_AST = null;
			if (inputState.guessing==0) {
				tmp144_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp144_AST);
			}
			match(LITERAL_finally);
			compoundStatement();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case FINAL:
		case ABSTRACT:
		case STRICTFP:
		case IDENT:
		case LCURLY:
		case RCURLY:
		case LPAREN:
		case SEMI:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case LITERAL_private:
		case LITERAL_public:
		case LITERAL_protected:
		case LITERAL_static:
		case LITERAL_transient:
		case LITERAL_native:
		case LITERAL_threadsafe:
		case LITERAL_synchronized:
		case LITERAL_volatile:
		case LITERAL_class:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_if:
		case LITERAL_else:
		case LITERAL_for:
		case LITERAL_while:
		case LITERAL_do:
		case LITERAL_break:
		case LITERAL_continue:
		case LITERAL_return:
		case LITERAL_switch:
		case LITERAL_throw:
		case LITERAL_case:
		case LITERAL_default:
		case LITERAL_try:
		case PLUS:
		case MINUS:
		case INC:
		case DEC:
		case BNOT:
		case LNOT:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		tryBlock_AST = (AST)currentAST.root;
		returnAST = tryBlock_AST;
	}
	
	public final void aCase() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST aCase_AST = null;
		
		{
		switch ( LA(1)) {
		case LITERAL_case:
		{
			AST tmp145_AST = null;
			if (inputState.guessing==0) {
				tmp145_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp145_AST);
			}
			match(LITERAL_case);
			expression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case LITERAL_default:
		{
			AST tmp146_AST = null;
			if (inputState.guessing==0) {
				tmp146_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp146_AST);
			}
			match(LITERAL_default);
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		AST tmp147_AST = null;
		tmp147_AST = (AST)astFactory.create(LT(1));
		match(COLON);
		aCase_AST = (AST)currentAST.root;
		returnAST = aCase_AST;
	}
	
	public final void caseSList() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST caseSList_AST = null;
		
		{
		_loop123:
		do {
			if ((_tokenSet_6.member(LA(1)))) {
				statement();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop123;
			}
			
		} while (true);
		}
		if ( inputState.guessing==0 ) {
			caseSList_AST = (AST)currentAST.root;
			caseSList_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(SLIST,"SLIST")).add(caseSList_AST));
			currentAST.root = caseSList_AST;
			currentAST.child = caseSList_AST!=null &&caseSList_AST.getFirstChild()!=null ?
				caseSList_AST.getFirstChild() : caseSList_AST;
			currentAST.advanceChildToEnd();
		}
		caseSList_AST = (AST)currentAST.root;
		returnAST = caseSList_AST;
	}
	
	public final void expressionList() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST expressionList_AST = null;
		
		expression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop140:
		do {
			if ((LA(1)==COMMA)) {
				AST tmp148_AST = null;
				tmp148_AST = (AST)astFactory.create(LT(1));
				match(COMMA);
				expression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop140;
			}
			
		} while (true);
		}
		if ( inputState.guessing==0 ) {
			expressionList_AST = (AST)currentAST.root;
			expressionList_AST = (AST)astFactory.make( (new ASTArray(2)).add((AST)astFactory.create(ELIST,"ELIST")).add(expressionList_AST));
			currentAST.root = expressionList_AST;
			currentAST.child = expressionList_AST!=null &&expressionList_AST.getFirstChild()!=null ?
				expressionList_AST.getFirstChild() : expressionList_AST;
			currentAST.advanceChildToEnd();
		}
		expressionList_AST = (AST)currentAST.root;
		returnAST = expressionList_AST;
	}
	
	public final void handler() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST handler_AST = null;
		
		AST tmp149_AST = null;
		if (inputState.guessing==0) {
			tmp149_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp149_AST);
		}
		match(LITERAL_catch);
		AST tmp150_AST = null;
		tmp150_AST = (AST)astFactory.create(LT(1));
		match(LPAREN);
		parameterDeclaration();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		AST tmp151_AST = null;
		tmp151_AST = (AST)astFactory.create(LT(1));
		match(RPAREN);
		compoundStatement();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		handler_AST = (AST)currentAST.root;
		returnAST = handler_AST;
	}
	
	public final void assignmentExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST assignmentExpression_AST = null;
		
		conditionalExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		switch ( LA(1)) {
		case ASSIGN:
		case PLUS_ASSIGN:
		case MINUS_ASSIGN:
		case STAR_ASSIGN:
		case DIV_ASSIGN:
		case MOD_ASSIGN:
		case SR_ASSIGN:
		case BSR_ASSIGN:
		case SL_ASSIGN:
		case BAND_ASSIGN:
		case BXOR_ASSIGN:
		case BOR_ASSIGN:
		{
			{
			switch ( LA(1)) {
			case ASSIGN:
			{
				AST tmp152_AST = null;
				if (inputState.guessing==0) {
					tmp152_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp152_AST);
				}
				match(ASSIGN);
				break;
			}
			case PLUS_ASSIGN:
			{
				AST tmp153_AST = null;
				if (inputState.guessing==0) {
					tmp153_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp153_AST);
				}
				match(PLUS_ASSIGN);
				break;
			}
			case MINUS_ASSIGN:
			{
				AST tmp154_AST = null;
				if (inputState.guessing==0) {
					tmp154_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp154_AST);
				}
				match(MINUS_ASSIGN);
				break;
			}
			case STAR_ASSIGN:
			{
				AST tmp155_AST = null;
				if (inputState.guessing==0) {
					tmp155_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp155_AST);
				}
				match(STAR_ASSIGN);
				break;
			}
			case DIV_ASSIGN:
			{
				AST tmp156_AST = null;
				if (inputState.guessing==0) {
					tmp156_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp156_AST);
				}
				match(DIV_ASSIGN);
				break;
			}
			case MOD_ASSIGN:
			{
				AST tmp157_AST = null;
				if (inputState.guessing==0) {
					tmp157_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp157_AST);
				}
				match(MOD_ASSIGN);
				break;
			}
			case SR_ASSIGN:
			{
				AST tmp158_AST = null;
				if (inputState.guessing==0) {
					tmp158_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp158_AST);
				}
				match(SR_ASSIGN);
				break;
			}
			case BSR_ASSIGN:
			{
				AST tmp159_AST = null;
				if (inputState.guessing==0) {
					tmp159_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp159_AST);
				}
				match(BSR_ASSIGN);
				break;
			}
			case SL_ASSIGN:
			{
				AST tmp160_AST = null;
				if (inputState.guessing==0) {
					tmp160_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp160_AST);
				}
				match(SL_ASSIGN);
				break;
			}
			case BAND_ASSIGN:
			{
				AST tmp161_AST = null;
				if (inputState.guessing==0) {
					tmp161_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp161_AST);
				}
				match(BAND_ASSIGN);
				break;
			}
			case BXOR_ASSIGN:
			{
				AST tmp162_AST = null;
				if (inputState.guessing==0) {
					tmp162_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp162_AST);
				}
				match(BXOR_ASSIGN);
				break;
			}
			case BOR_ASSIGN:
			{
				AST tmp163_AST = null;
				if (inputState.guessing==0) {
					tmp163_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp163_AST);
				}
				match(BOR_ASSIGN);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			assignmentExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case RCURLY:
		case RPAREN:
		case SEMI:
		case RBRACK:
		case COMMA:
		case COLON:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		assignmentExpression_AST = (AST)currentAST.root;
		returnAST = assignmentExpression_AST;
	}
	
	public final void conditionalExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST conditionalExpression_AST = null;
		
		logicalOrExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		switch ( LA(1)) {
		case QUESTION:
		{
			AST tmp164_AST = null;
			if (inputState.guessing==0) {
				tmp164_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp164_AST);
			}
			match(QUESTION);
			assignmentExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp165_AST = null;
			tmp165_AST = (AST)astFactory.create(LT(1));
			match(COLON);
			conditionalExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		case RCURLY:
		case RPAREN:
		case SEMI:
		case ASSIGN:
		case RBRACK:
		case COMMA:
		case COLON:
		case PLUS_ASSIGN:
		case MINUS_ASSIGN:
		case STAR_ASSIGN:
		case DIV_ASSIGN:
		case MOD_ASSIGN:
		case SR_ASSIGN:
		case BSR_ASSIGN:
		case SL_ASSIGN:
		case BAND_ASSIGN:
		case BXOR_ASSIGN:
		case BOR_ASSIGN:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		conditionalExpression_AST = (AST)currentAST.root;
		returnAST = conditionalExpression_AST;
	}
	
	public final void logicalOrExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST logicalOrExpression_AST = null;
		
		logicalAndExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop148:
		do {
			if ((LA(1)==LOR)) {
				AST tmp166_AST = null;
				if (inputState.guessing==0) {
					tmp166_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp166_AST);
				}
				match(LOR);
				logicalAndExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop148;
			}
			
		} while (true);
		}
		logicalOrExpression_AST = (AST)currentAST.root;
		returnAST = logicalOrExpression_AST;
	}
	
	public final void logicalAndExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST logicalAndExpression_AST = null;
		
		inclusiveOrExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop151:
		do {
			if ((LA(1)==LAND)) {
				AST tmp167_AST = null;
				if (inputState.guessing==0) {
					tmp167_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp167_AST);
				}
				match(LAND);
				inclusiveOrExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop151;
			}
			
		} while (true);
		}
		logicalAndExpression_AST = (AST)currentAST.root;
		returnAST = logicalAndExpression_AST;
	}
	
	public final void exclusiveOrExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST exclusiveOrExpression_AST = null;
		
		andExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop157:
		do {
			if ((LA(1)==BXOR)) {
				AST tmp168_AST = null;
				if (inputState.guessing==0) {
					tmp168_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp168_AST);
				}
				match(BXOR);
				andExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop157;
			}
			
		} while (true);
		}
		exclusiveOrExpression_AST = (AST)currentAST.root;
		returnAST = exclusiveOrExpression_AST;
	}
	
	public final void andExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST andExpression_AST = null;
		
		equalityExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop160:
		do {
			if ((LA(1)==BAND)) {
				AST tmp169_AST = null;
				if (inputState.guessing==0) {
					tmp169_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp169_AST);
				}
				match(BAND);
				equalityExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop160;
			}
			
		} while (true);
		}
		andExpression_AST = (AST)currentAST.root;
		returnAST = andExpression_AST;
	}
	
	public final void equalityExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST equalityExpression_AST = null;
		
		relationalExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop164:
		do {
			if ((LA(1)==NOT_EQUAL||LA(1)==EQUAL)) {
				{
				switch ( LA(1)) {
				case NOT_EQUAL:
				{
					AST tmp170_AST = null;
					if (inputState.guessing==0) {
						tmp170_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp170_AST);
					}
					match(NOT_EQUAL);
					break;
				}
				case EQUAL:
				{
					AST tmp171_AST = null;
					if (inputState.guessing==0) {
						tmp171_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp171_AST);
					}
					match(EQUAL);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				relationalExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop164;
			}
			
		} while (true);
		}
		equalityExpression_AST = (AST)currentAST.root;
		returnAST = equalityExpression_AST;
	}
	
	public final void relationalExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST relationalExpression_AST = null;
		
		shiftExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		switch ( LA(1)) {
		case EOF:
		case RCURLY:
		case RPAREN:
		case SEMI:
		case ASSIGN:
		case RBRACK:
		case COMMA:
		case COLON:
		case PLUS_ASSIGN:
		case MINUS_ASSIGN:
		case STAR_ASSIGN:
		case DIV_ASSIGN:
		case MOD_ASSIGN:
		case SR_ASSIGN:
		case BSR_ASSIGN:
		case SL_ASSIGN:
		case BAND_ASSIGN:
		case BXOR_ASSIGN:
		case BOR_ASSIGN:
		case QUESTION:
		case LOR:
		case LAND:
		case BOR:
		case BXOR:
		case BAND:
		case NOT_EQUAL:
		case EQUAL:
		case LT:
		case GT:
		case LE:
		case GE:
		{
			{
			_loop169:
			do {
				if (((LA(1) >= LT && LA(1) <= GE))) {
					{
					switch ( LA(1)) {
					case LT:
					{
						AST tmp172_AST = null;
						if (inputState.guessing==0) {
							tmp172_AST = (AST)astFactory.create(LT(1));
							astFactory.makeASTRoot(currentAST, tmp172_AST);
						}
						match(LT);
						break;
					}
					case GT:
					{
						AST tmp173_AST = null;
						if (inputState.guessing==0) {
							tmp173_AST = (AST)astFactory.create(LT(1));
							astFactory.makeASTRoot(currentAST, tmp173_AST);
						}
						match(GT);
						break;
					}
					case LE:
					{
						AST tmp174_AST = null;
						if (inputState.guessing==0) {
							tmp174_AST = (AST)astFactory.create(LT(1));
							astFactory.makeASTRoot(currentAST, tmp174_AST);
						}
						match(LE);
						break;
					}
					case GE:
					{
						AST tmp175_AST = null;
						if (inputState.guessing==0) {
							tmp175_AST = (AST)astFactory.create(LT(1));
							astFactory.makeASTRoot(currentAST, tmp175_AST);
						}
						match(GE);
						break;
					}
					default:
					{
						throw new NoViableAltException(LT(1), getFilename());
					}
					}
					}
					shiftExpression();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else {
					break _loop169;
				}
				
			} while (true);
			}
			break;
		}
		case LITERAL_instanceof:
		{
			AST tmp176_AST = null;
			if (inputState.guessing==0) {
				tmp176_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp176_AST);
			}
			match(LITERAL_instanceof);
			typeSpec(true);
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		relationalExpression_AST = (AST)currentAST.root;
		returnAST = relationalExpression_AST;
	}
	
	public final void shiftExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST shiftExpression_AST = null;
		
		additiveExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop173:
		do {
			if (((LA(1) >= SL && LA(1) <= BSR))) {
				{
				switch ( LA(1)) {
				case SL:
				{
					AST tmp177_AST = null;
					if (inputState.guessing==0) {
						tmp177_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp177_AST);
					}
					match(SL);
					break;
				}
				case SR:
				{
					AST tmp178_AST = null;
					if (inputState.guessing==0) {
						tmp178_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp178_AST);
					}
					match(SR);
					break;
				}
				case BSR:
				{
					AST tmp179_AST = null;
					if (inputState.guessing==0) {
						tmp179_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp179_AST);
					}
					match(BSR);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				additiveExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop173;
			}
			
		} while (true);
		}
		shiftExpression_AST = (AST)currentAST.root;
		returnAST = shiftExpression_AST;
	}
	
	public final void additiveExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST additiveExpression_AST = null;
		
		multiplicativeExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop177:
		do {
			if ((LA(1)==PLUS||LA(1)==MINUS)) {
				{
				switch ( LA(1)) {
				case PLUS:
				{
					AST tmp180_AST = null;
					if (inputState.guessing==0) {
						tmp180_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp180_AST);
					}
					match(PLUS);
					break;
				}
				case MINUS:
				{
					AST tmp181_AST = null;
					if (inputState.guessing==0) {
						tmp181_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp181_AST);
					}
					match(MINUS);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				multiplicativeExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop177;
			}
			
		} while (true);
		}
		additiveExpression_AST = (AST)currentAST.root;
		returnAST = additiveExpression_AST;
	}
	
	public final void multiplicativeExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST multiplicativeExpression_AST = null;
		
		unaryExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop181:
		do {
			if ((_tokenSet_28.member(LA(1)))) {
				{
				switch ( LA(1)) {
				case STAR:
				{
					AST tmp182_AST = null;
					if (inputState.guessing==0) {
						tmp182_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp182_AST);
					}
					match(STAR);
					break;
				}
				case DIV:
				{
					AST tmp183_AST = null;
					if (inputState.guessing==0) {
						tmp183_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp183_AST);
					}
					match(DIV);
					break;
				}
				case MOD:
				{
					AST tmp184_AST = null;
					if (inputState.guessing==0) {
						tmp184_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp184_AST);
					}
					match(MOD);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				unaryExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				break _loop181;
			}
			
		} while (true);
		}
		multiplicativeExpression_AST = (AST)currentAST.root;
		returnAST = multiplicativeExpression_AST;
	}
	
	public final void unaryExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST unaryExpression_AST = null;
		
		switch ( LA(1)) {
		case INC:
		{
			AST tmp185_AST = null;
			if (inputState.guessing==0) {
				tmp185_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp185_AST);
			}
			match(INC);
			unaryExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			unaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case DEC:
		{
			AST tmp186_AST = null;
			if (inputState.guessing==0) {
				tmp186_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp186_AST);
			}
			match(DEC);
			unaryExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			unaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case MINUS:
		{
			AST tmp187_AST = null;
			if (inputState.guessing==0) {
				tmp187_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp187_AST);
			}
			match(MINUS);
			if ( inputState.guessing==0 ) {
				tmp187_AST.setType(UNARY_MINUS);
			}
			unaryExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			unaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case PLUS:
		{
			AST tmp188_AST = null;
			if (inputState.guessing==0) {
				tmp188_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp188_AST);
			}
			match(PLUS);
			if ( inputState.guessing==0 ) {
				tmp188_AST.setType(UNARY_PLUS);
			}
			unaryExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			unaryExpression_AST = (AST)currentAST.root;
			break;
		}
		case IDENT:
		case LPAREN:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case LITERAL_this:
		case LITERAL_super:
		case BNOT:
		case LNOT:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			unaryExpressionNotPlusMinus();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			unaryExpression_AST = (AST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = unaryExpression_AST;
	}
	
	public final void unaryExpressionNotPlusMinus() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST unaryExpressionNotPlusMinus_AST = null;
		Token  lpb = null;
		AST lpb_AST = null;
		Token  lp = null;
		AST lp_AST = null;
		
		switch ( LA(1)) {
		case BNOT:
		{
			AST tmp189_AST = null;
			if (inputState.guessing==0) {
				tmp189_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp189_AST);
			}
			match(BNOT);
			unaryExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			unaryExpressionNotPlusMinus_AST = (AST)currentAST.root;
			break;
		}
		case LNOT:
		{
			AST tmp190_AST = null;
			if (inputState.guessing==0) {
				tmp190_AST = (AST)astFactory.create(LT(1));
				astFactory.makeASTRoot(currentAST, tmp190_AST);
			}
			match(LNOT);
			unaryExpression();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			unaryExpressionNotPlusMinus_AST = (AST)currentAST.root;
			break;
		}
		case IDENT:
		case LPAREN:
		case LITERAL_void:
		case LITERAL_boolean:
		case LITERAL_byte:
		case LITERAL_char:
		case LITERAL_short:
		case LITERAL_int:
		case LITERAL_float:
		case LITERAL_long:
		case LITERAL_double:
		case LITERAL_this:
		case LITERAL_super:
		case LITERAL_true:
		case LITERAL_false:
		case LITERAL_null:
		case LITERAL_new:
		case NUM_INT:
		case CHAR_LITERAL:
		case STRING_LITERAL:
		case NUM_FLOAT:
		case NUM_LONG:
		case NUM_DOUBLE:
		{
			{
			if ((LA(1)==LPAREN) && ((LA(2) >= LITERAL_void && LA(2) <= LITERAL_double))) {
				lpb = LT(1);
				if (inputState.guessing==0) {
					lpb_AST = (AST)astFactory.create(lpb);
					astFactory.makeASTRoot(currentAST, lpb_AST);
				}
				match(LPAREN);
				if ( inputState.guessing==0 ) {
					lpb_AST.setType(TYPECAST);
				}
				builtInTypeSpec(true);
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				AST tmp191_AST = null;
				tmp191_AST = (AST)astFactory.create(LT(1));
				match(RPAREN);
				unaryExpression();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
			}
			else {
				boolean synPredMatched186 = false;
				if (((LA(1)==LPAREN) && (LA(2)==IDENT))) {
					int _m186 = mark();
					synPredMatched186 = true;
					inputState.guessing++;
					try {
						{
						match(LPAREN);
						classTypeSpec(true);
						match(RPAREN);
						unaryExpressionNotPlusMinus();
						}
					}
					catch (RecognitionException pe) {
						synPredMatched186 = false;
					}
					rewind(_m186);
					inputState.guessing--;
				}
				if ( synPredMatched186 ) {
					lp = LT(1);
					if (inputState.guessing==0) {
						lp_AST = (AST)astFactory.create(lp);
						astFactory.makeASTRoot(currentAST, lp_AST);
					}
					match(LPAREN);
					if ( inputState.guessing==0 ) {
						lp_AST.setType(TYPECAST);
					}
					classTypeSpec(true);
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
					AST tmp192_AST = null;
					tmp192_AST = (AST)astFactory.create(LT(1));
					match(RPAREN);
					unaryExpressionNotPlusMinus();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else if ((_tokenSet_14.member(LA(1))) && (_tokenSet_29.member(LA(2)))) {
					postfixExpression();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
				}
				else {
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				unaryExpressionNotPlusMinus_AST = (AST)currentAST.root;
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			returnAST = unaryExpressionNotPlusMinus_AST;
		}
		
	public final void postfixExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST postfixExpression_AST = null;
		Token  lbc = null;
		AST lbc_AST = null;
		Token  lb = null;
		AST lb_AST = null;
		Token  lp = null;
		AST lp_AST = null;
		Token  in = null;
		AST in_AST = null;
		Token  de = null;
		AST de_AST = null;
		
		primaryExpression();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		_loop192:
		do {
			switch ( LA(1)) {
			case DOT:
			{
				AST tmp193_AST = null;
				if (inputState.guessing==0) {
					tmp193_AST = (AST)astFactory.create(LT(1));
					astFactory.makeASTRoot(currentAST, tmp193_AST);
				}
				match(DOT);
				{
				switch ( LA(1)) {
				case IDENT:
				{
					AST tmp194_AST = null;
					if (inputState.guessing==0) {
						tmp194_AST = (AST)astFactory.create(LT(1));
						astFactory.addASTChild(currentAST, tmp194_AST);
					}
					match(IDENT);
					break;
				}
				case LITERAL_this:
				{
					AST tmp195_AST = null;
					if (inputState.guessing==0) {
						tmp195_AST = (AST)astFactory.create(LT(1));
						astFactory.addASTChild(currentAST, tmp195_AST);
					}
					match(LITERAL_this);
					break;
				}
				case LITERAL_class:
				{
					AST tmp196_AST = null;
					if (inputState.guessing==0) {
						tmp196_AST = (AST)astFactory.create(LT(1));
						astFactory.addASTChild(currentAST, tmp196_AST);
					}
					match(LITERAL_class);
					break;
				}
				case LITERAL_new:
				{
					newExpression();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
					break;
				}
				case LITERAL_super:
				{
					AST tmp197_AST = null;
					if (inputState.guessing==0) {
						tmp197_AST = (AST)astFactory.create(LT(1));
						astFactory.addASTChild(currentAST, tmp197_AST);
					}
					match(LITERAL_super);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case LPAREN:
			{
				lp = LT(1);
				if (inputState.guessing==0) {
					lp_AST = (AST)astFactory.create(lp);
					astFactory.makeASTRoot(currentAST, lp_AST);
				}
				match(LPAREN);
				if ( inputState.guessing==0 ) {
					lp_AST.setType(METHOD_CALL);
				}
				argList();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				AST tmp198_AST = null;
				tmp198_AST = (AST)astFactory.create(LT(1));
				match(RPAREN);
				break;
			}
			default:
				if ((LA(1)==LBRACK) && (LA(2)==RBRACK)) {
					{
					int _cnt191=0;
					_loop191:
					do {
						if ((LA(1)==LBRACK)) {
							lbc = LT(1);
							if (inputState.guessing==0) {
								lbc_AST = (AST)astFactory.create(lbc);
								astFactory.makeASTRoot(currentAST, lbc_AST);
							}
							match(LBRACK);
							if ( inputState.guessing==0 ) {
								lbc_AST.setType(ARRAY_DECLARATOR);
							}
							AST tmp199_AST = null;
							tmp199_AST = (AST)astFactory.create(LT(1));
							match(RBRACK);
						}
						else {
							if ( _cnt191>=1 ) { break _loop191; } else {throw new NoViableAltException(LT(1), getFilename());}
						}
						
						_cnt191++;
					} while (true);
					}
					AST tmp200_AST = null;
					if (inputState.guessing==0) {
						tmp200_AST = (AST)astFactory.create(LT(1));
						astFactory.makeASTRoot(currentAST, tmp200_AST);
					}
					match(DOT);
					AST tmp201_AST = null;
					if (inputState.guessing==0) {
						tmp201_AST = (AST)astFactory.create(LT(1));
						astFactory.addASTChild(currentAST, tmp201_AST);
					}
					match(LITERAL_class);
				}
				else if ((LA(1)==LBRACK) && (_tokenSet_4.member(LA(2)))) {
					lb = LT(1);
					if (inputState.guessing==0) {
						lb_AST = (AST)astFactory.create(lb);
						astFactory.makeASTRoot(currentAST, lb_AST);
					}
					match(LBRACK);
					if ( inputState.guessing==0 ) {
						lb_AST.setType(INDEX_OP);
					}
					expression();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
					AST tmp202_AST = null;
					tmp202_AST = (AST)astFactory.create(LT(1));
					match(RBRACK);
				}
			else {
				break _loop192;
			}
			}
		} while (true);
		}
		{
		switch ( LA(1)) {
		case INC:
		{
			in = LT(1);
			if (inputState.guessing==0) {
				in_AST = (AST)astFactory.create(in);
				astFactory.makeASTRoot(currentAST, in_AST);
			}
			match(INC);
			if ( inputState.guessing==0 ) {
				in_AST.setType(POST_INC);
			}
			break;
		}
		case DEC:
		{
			de = LT(1);
			if (inputState.guessing==0) {
				de_AST = (AST)astFactory.create(de);
				astFactory.makeASTRoot(currentAST, de_AST);
			}
			match(DEC);
			if ( inputState.guessing==0 ) {
				de_AST.setType(POST_DEC);
			}
			break;
		}
		case EOF:
		case RCURLY:
		case RPAREN:
		case SEMI:
		case ASSIGN:
		case RBRACK:
		case STAR:
		case COMMA:
		case COLON:
		case PLUS_ASSIGN:
		case MINUS_ASSIGN:
		case STAR_ASSIGN:
		case DIV_ASSIGN:
		case MOD_ASSIGN:
		case SR_ASSIGN:
		case BSR_ASSIGN:
		case SL_ASSIGN:
		case BAND_ASSIGN:
		case BXOR_ASSIGN:
		case BOR_ASSIGN:
		case QUESTION:
		case LOR:
		case LAND:
		case BOR:
		case BXOR:
		case BAND:
		case NOT_EQUAL:
		case EQUAL:
		case LT:
		case GT:
		case LE:
		case GE:
		case LITERAL_instanceof:
		case SL:
		case SR:
		case BSR:
		case PLUS:
		case MINUS:
		case DIV:
		case MOD:
		{
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		postfixExpression_AST = (AST)currentAST.root;
		returnAST = postfixExpression_AST;
	}
	
/** object instantiation.
 *  Trees are built as illustrated by the following input/tree pairs:
 *  
 *  new T()
 *  
 *  new
 *   |
 *   T --  ELIST
 *           |
 *          arg1 -- arg2 -- .. -- argn
 *  
 *  new int[]
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *  
 *  new int[] {1,2}
 *
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR -- ARRAY_INIT
 *                                  |
 *                                EXPR -- EXPR
 *                                  |      |
 *                                  1      2
 *  
 *  new int[3]
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *                |
 *              EXPR
 *                |
 *                3
 *  
 *  new int[1][2]
 *  
 *  new
 *   |
 *  int -- ARRAY_DECLARATOR
 *               |
 *         ARRAY_DECLARATOR -- EXPR
 *               |              |
 *             EXPR             1
 *               |
 *               2
 *  
 */
	public final void newExpression() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST newExpression_AST = null;
		
		AST tmp203_AST = null;
		if (inputState.guessing==0) {
			tmp203_AST = (AST)astFactory.create(LT(1));
			astFactory.makeASTRoot(currentAST, tmp203_AST);
		}
		match(LITERAL_new);
		type();
		if (inputState.guessing==0) {
			astFactory.addASTChild(currentAST, returnAST);
		}
		{
		switch ( LA(1)) {
		case LPAREN:
		{
			AST tmp204_AST = null;
			tmp204_AST = (AST)astFactory.create(LT(1));
			match(LPAREN);
			argList();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			AST tmp205_AST = null;
			tmp205_AST = (AST)astFactory.create(LT(1));
			match(RPAREN);
			{
			switch ( LA(1)) {
			case LCURLY:
			{
				classBlock();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				break;
			}
			case EOF:
			case RCURLY:
			case LPAREN:
			case RPAREN:
			case SEMI:
			case ASSIGN:
			case LBRACK:
			case RBRACK:
			case DOT:
			case STAR:
			case COMMA:
			case COLON:
			case PLUS_ASSIGN:
			case MINUS_ASSIGN:
			case STAR_ASSIGN:
			case DIV_ASSIGN:
			case MOD_ASSIGN:
			case SR_ASSIGN:
			case BSR_ASSIGN:
			case SL_ASSIGN:
			case BAND_ASSIGN:
			case BXOR_ASSIGN:
			case BOR_ASSIGN:
			case QUESTION:
			case LOR:
			case LAND:
			case BOR:
			case BXOR:
			case BAND:
			case NOT_EQUAL:
			case EQUAL:
			case LT:
			case GT:
			case LE:
			case GE:
			case LITERAL_instanceof:
			case SL:
			case SR:
			case BSR:
			case PLUS:
			case MINUS:
			case DIV:
			case MOD:
			case INC:
			case DEC:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		case LBRACK:
		{
			newArrayDeclarator();
			if (inputState.guessing==0) {
				astFactory.addASTChild(currentAST, returnAST);
			}
			{
			switch ( LA(1)) {
			case LCURLY:
			{
				arrayInitializer();
				if (inputState.guessing==0) {
					astFactory.addASTChild(currentAST, returnAST);
				}
				break;
			}
			case EOF:
			case RCURLY:
			case LPAREN:
			case RPAREN:
			case SEMI:
			case ASSIGN:
			case LBRACK:
			case RBRACK:
			case DOT:
			case STAR:
			case COMMA:
			case COLON:
			case PLUS_ASSIGN:
			case MINUS_ASSIGN:
			case STAR_ASSIGN:
			case DIV_ASSIGN:
			case MOD_ASSIGN:
			case SR_ASSIGN:
			case BSR_ASSIGN:
			case SL_ASSIGN:
			case BAND_ASSIGN:
			case BXOR_ASSIGN:
			case BOR_ASSIGN:
			case QUESTION:
			case LOR:
			case LAND:
			case BOR:
			case BXOR:
			case BAND:
			case NOT_EQUAL:
			case EQUAL:
			case LT:
			case GT:
			case LE:
			case GE:
			case LITERAL_instanceof:
			case SL:
			case SR:
			case BSR:
			case PLUS:
			case MINUS:
			case DIV:
			case MOD:
			case INC:
			case DEC:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		}
		newExpression_AST = (AST)currentAST.root;
		returnAST = newExpression_AST;
	}
	
	public final void constant() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST constant_AST = null;
		
		switch ( LA(1)) {
		case NUM_INT:
		{
			AST tmp206_AST = null;
			if (inputState.guessing==0) {
				tmp206_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp206_AST);
			}
			match(NUM_INT);
			constant_AST = (AST)currentAST.root;
			break;
		}
		case CHAR_LITERAL:
		{
			AST tmp207_AST = null;
			if (inputState.guessing==0) {
				tmp207_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp207_AST);
			}
			match(CHAR_LITERAL);
			constant_AST = (AST)currentAST.root;
			break;
		}
		case STRING_LITERAL:
		{
			AST tmp208_AST = null;
			if (inputState.guessing==0) {
				tmp208_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp208_AST);
			}
			match(STRING_LITERAL);
			constant_AST = (AST)currentAST.root;
			break;
		}
		case NUM_FLOAT:
		{
			AST tmp209_AST = null;
			if (inputState.guessing==0) {
				tmp209_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp209_AST);
			}
			match(NUM_FLOAT);
			constant_AST = (AST)currentAST.root;
			break;
		}
		case NUM_LONG:
		{
			AST tmp210_AST = null;
			if (inputState.guessing==0) {
				tmp210_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp210_AST);
			}
			match(NUM_LONG);
			constant_AST = (AST)currentAST.root;
			break;
		}
		case NUM_DOUBLE:
		{
			AST tmp211_AST = null;
			if (inputState.guessing==0) {
				tmp211_AST = (AST)astFactory.create(LT(1));
				astFactory.addASTChild(currentAST, tmp211_AST);
			}
			match(NUM_DOUBLE);
			constant_AST = (AST)currentAST.root;
			break;
		}
		default:
		{
			throw new NoViableAltException(LT(1), getFilename());
		}
		}
		returnAST = constant_AST;
	}
	
	public final void newArrayDeclarator() throws RecognitionException, TokenStreamException {
		
		returnAST = null;
		ASTPair currentAST = new ASTPair();
		AST newArrayDeclarator_AST = null;
		Token  lb = null;
		AST lb_AST = null;
		
		{
		int _cnt206=0;
		_loop206:
		do {
			if ((LA(1)==LBRACK) && (_tokenSet_30.member(LA(2)))) {
				lb = LT(1);
				if (inputState.guessing==0) {
					lb_AST = (AST)astFactory.create(lb);
					astFactory.makeASTRoot(currentAST, lb_AST);
				}
				match(LBRACK);
				if ( inputState.guessing==0 ) {
					lb_AST.setType(ARRAY_DECLARATOR);
				}
				{
				switch ( LA(1)) {
				case IDENT:
				case LPAREN:
				case LITERAL_void:
				case LITERAL_boolean:
				case LITERAL_byte:
				case LITERAL_char:
				case LITERAL_short:
				case LITERAL_int:
				case LITERAL_float:
				case LITERAL_long:
				case LITERAL_double:
				case LITERAL_this:
				case LITERAL_super:
				case PLUS:
				case MINUS:
				case INC:
				case DEC:
				case BNOT:
				case LNOT:
				case LITERAL_true:
				case LITERAL_false:
				case LITERAL_null:
				case LITERAL_new:
				case NUM_INT:
				case CHAR_LITERAL:
				case STRING_LITERAL:
				case NUM_FLOAT:
				case NUM_LONG:
				case NUM_DOUBLE:
				{
					expression();
					if (inputState.guessing==0) {
						astFactory.addASTChild(currentAST, returnAST);
					}
					break;
				}
				case RBRACK:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				AST tmp212_AST = null;
				tmp212_AST = (AST)astFactory.create(LT(1));
				match(RBRACK);
			}
			else {
				if ( _cnt206>=1 ) { break _loop206; } else {throw new NoViableAltException(LT(1), getFilename());}
			}
			
			_cnt206++;
		} while (true);
		}
		newArrayDeclarator_AST = (AST)currentAST.root;
		returnAST = newArrayDeclarator_AST;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"BLOCK",
		"MODIFIERS",
		"OBJBLOCK",
		"SLIST",
		"CTOR_DEF",
		"METHOD_DEF",
		"VARIABLE_DEF",
		"INSTANCE_INIT",
		"STATIC_INIT",
		"TYPE",
		"CLASS_DEF",
		"INTERFACE_DEF",
		"PACKAGE_DEF",
		"ARRAY_DECLARATOR",
		"EXTENDS_CLAUSE",
		"IMPLEMENTS_CLAUSE",
		"PARAMETERS",
		"PARAMETER_DEF",
		"LABELED_STAT",
		"TYPECAST",
		"INDEX_OP",
		"POST_INC",
		"POST_DEC",
		"METHOD_CALL",
		"EXPR",
		"ARRAY_INIT",
		"IMPORT",
		"UNARY_MINUS",
		"UNARY_PLUS",
		"CASE_GROUP",
		"ELIST",
		"FOR_INIT",
		"FOR_CONDITION",
		"FOR_ITERATOR",
		"EMPTY_STAT",
		"\"final\"",
		"\"abstract\"",
		"\"strictfp\"",
		"SUPER_CTOR_CALL",
		"CTOR_CALL",
		"\"ruleset\"",
		"\"rule\"",
		"\"when\"",
		"\"then\"",
		"IDENT",
		"LCURLY",
		"RCURLY",
		"LPAREN",
		"RPAREN",
		"SEMI",
		"ASSIGN",
		"\"package\"",
		"\"import\"",
		"LBRACK",
		"RBRACK",
		"\"void\"",
		"\"boolean\"",
		"\"byte\"",
		"\"char\"",
		"\"short\"",
		"\"int\"",
		"\"float\"",
		"\"long\"",
		"\"double\"",
		"DOT",
		"STAR",
		"\"private\"",
		"\"public\"",
		"\"protected\"",
		"\"static\"",
		"\"transient\"",
		"\"native\"",
		"\"threadsafe\"",
		"\"synchronized\"",
		"\"volatile\"",
		"\"class\"",
		"\"extends\"",
		"\"interface\"",
		"COMMA",
		"\"implements\"",
		"\"this\"",
		"\"super\"",
		"\"throws\"",
		"COLON",
		"\"if\"",
		"\"else\"",
		"\"for\"",
		"\"while\"",
		"\"do\"",
		"\"break\"",
		"\"continue\"",
		"\"return\"",
		"\"switch\"",
		"\"throw\"",
		"\"case\"",
		"\"default\"",
		"\"try\"",
		"\"finally\"",
		"\"catch\"",
		"PLUS_ASSIGN",
		"MINUS_ASSIGN",
		"STAR_ASSIGN",
		"DIV_ASSIGN",
		"MOD_ASSIGN",
		"SR_ASSIGN",
		"BSR_ASSIGN",
		"SL_ASSIGN",
		"BAND_ASSIGN",
		"BXOR_ASSIGN",
		"BOR_ASSIGN",
		"QUESTION",
		"LOR",
		"LAND",
		"BOR",
		"BXOR",
		"BAND",
		"NOT_EQUAL",
		"EQUAL",
		"LT",
		"GT",
		"LE",
		"GE",
		"\"instanceof\"",
		"SL",
		"SR",
		"BSR",
		"PLUS",
		"MINUS",
		"DIV",
		"MOD",
		"INC",
		"DEC",
		"BNOT",
		"LNOT",
		"\"true\"",
		"\"false\"",
		"\"null\"",
		"\"new\"",
		"NUM_INT",
		"CHAR_LITERAL",
		"STRING_LITERAL",
		"NUM_FLOAT",
		"NUM_LONG",
		"NUM_DOUBLE",
		"WS",
		"SL_COMMENT",
		"ML_COMMENT",
		"ESC",
		"HEX_DIGIT",
		"VOCAB",
		"EXPONENT",
		"FLOAT_SUFFIX"
	};
	
	private static final long _tokenSet_0_data_[] = { 9011047545438208L, 196544L, 0L, 0L };
	public static final BitSet _tokenSet_0 = new BitSet(_tokenSet_0_data_);
	private static final long _tokenSet_1_data_[] = { 81086233769410562L, 196544L, 0L, 0L };
	public static final BitSet _tokenSet_1 = new BitSet(_tokenSet_1_data_);
	private static final long _tokenSet_2_data_[] = { 9011047545438210L, 196544L, 0L, 0L };
	public static final BitSet _tokenSet_2 = new BitSet(_tokenSet_2_data_);
	private static final long _tokenSet_3_data_[] = { -576178727570898944L, 15L, 0L, 0L };
	public static final BitSet _tokenSet_3 = new BitSet(_tokenSet_3_data_);
	private static final long _tokenSet_4_data_[] = { -573927477513027584L, 3145743L, 1048524L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_4 = new BitSet(_tokenSet_4_data_);
	private static final long _tokenSet_5_data_[] = { -420805090182430720L, -9007199251595201L, 1048575L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_5 = new BitSet(_tokenSet_5_data_);
	private static final long _tokenSet_6_data_[] = { -564353480014168064L, 85852225487L, 1048524L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_6 = new BitSet(_tokenSet_6_data_);
	private static final long _tokenSet_7_data_[] = { -429812289437171710L, -9007199251595201L, 1048575L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_7 = new BitSet(_tokenSet_7_data_);
	private static final long _tokenSet_8_data_[] = { 3848290697216L, 32704L, 0L, 0L };
	public static final BitSet _tokenSet_8 = new BitSet(_tokenSet_8_data_);
	private static final long _tokenSet_9_data_[] = { -576175429036015616L, 196559L, 0L, 0L };
	public static final BitSet _tokenSet_9 = new BitSet(_tokenSet_9_data_);
	private static final long _tokenSet_10_data_[] = { -429808441146474496L, 196575L, 0L, 0L };
	public static final BitSet _tokenSet_10 = new BitSet(_tokenSet_10_data_);
	private static final long _tokenSet_11_data_[] = { -576179277326712832L, 15L, 0L, 0L };
	public static final BitSet _tokenSet_11 = new BitSet(_tokenSet_11_data_);
	private static final long _tokenSet_12_data_[] = { 144396663052566528L, 16L, 0L, 0L };
	public static final BitSet _tokenSet_12 = new BitSet(_tokenSet_12_data_);
	private static final long _tokenSet_13_data_[] = { 171136785840078848L, 262144L, 0L, 0L };
	public static final BitSet _tokenSet_13 = new BitSet(_tokenSet_13_data_);
	private static final long _tokenSet_14_data_[] = { -573927477513027584L, 3145743L, 1047552L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_14 = new BitSet(_tokenSet_14_data_);
	private static final long _tokenSet_15_data_[] = { -429812289437171712L, 3145759L, 1048524L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_15 = new BitSet(_tokenSet_15_data_);
	private static final long _tokenSet_16_data_[] = { -563227580107325440L, 85852225487L, 1048524L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_16 = new BitSet(_tokenSet_16_data_);
	private static final long _tokenSet_17_data_[] = { -401097993521987584L, -463895068673L, 1048575L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_17 = new BitSet(_tokenSet_17_data_);
	private static final long _tokenSet_18_data_[] = { -563227580107325440L, 137425387471L, 1048524L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_18 = new BitSet(_tokenSet_18_data_);
	private static final long _tokenSet_19_data_[] = { -401097993521987584L, -5046273L, 1048575L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_19 = new BitSet(_tokenSet_19_data_);
	private static final long _tokenSet_20_data_[] = { -576175429036015616L, 32719L, 0L, 0L };
	public static final BitSet _tokenSet_20 = new BitSet(_tokenSet_20_data_);
	private static final long _tokenSet_21_data_[] = { -432060240960159744L, 32735L, 0L, 0L };
	public static final BitSet _tokenSet_21 = new BitSet(_tokenSet_21_data_);
	private static final long _tokenSet_22_data_[] = { -402790691672948736L, -549752668097L, 1048575L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_22 = new BitSet(_tokenSet_22_data_);
	private static final long _tokenSet_23_data_[] = { 3848290697216L, 65472L, 0L, 0L };
	public static final BitSet _tokenSet_23 = new BitSet(_tokenSet_23_data_);
	private static final long _tokenSet_24_data_[] = { 285323267407872L, 65472L, 0L, 0L };
	public static final BitSet _tokenSet_24 = new BitSet(_tokenSet_24_data_);
	private static final long _tokenSet_25_data_[] = { -573364527559606272L, 3145743L, 1048524L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_25 = new BitSet(_tokenSet_25_data_);
	private static final long _tokenSet_26_data_[] = { -402790691672948736L, -549752405953L, 1048575L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_26 = new BitSet(_tokenSet_26_data_);
	private static final long _tokenSet_27_data_[] = { -573927477513027584L, 11534351L, 1048524L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_27 = new BitSet(_tokenSet_27_data_);
	private static final long _tokenSet_28_data_[] = { 0L, 32L, 48L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_28 = new BitSet(_tokenSet_28_data_);
	private static final long _tokenSet_29_data_[] = { -108930815987023870L, -549744017345L, 1048575L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_29 = new BitSet(_tokenSet_29_data_);
	private static final long _tokenSet_30_data_[] = { -285697101361315840L, 3145743L, 1048524L, 0L, 0L, 0L };
	public static final BitSet _tokenSet_30 = new BitSet(_tokenSet_30_data_);
	
	}
