/*
 * $Id$
 *
 * Copyright 2002 John Pritchard, All Rights Reserved.
 *
 * Licensed under the "Common Public License 1.0".
 *  http://www.yajre.org/license.txt
 */

package org.yajre.semantic.java ;

import bsh.Interpreter;
import bsh.EvalError;

import org.yajre.* ;

import java.io.* ;

import java.util.* ;

/**
 * Command line testing shell for rules runtime implementors.
 * 
 * @author John Pritchard
 */
public class test extends Object {

    public final RuleBase rules = new RuleBase(true);

    public String ruleset = null;

    public WorkingMemory model = null;

    public Interpreter interp;

    public db debugger = null;

    public test(){
	super();
    }

    public test(Interpreter interp){
	super();
	this.interp = interp;
    }

    public void ruleset( String filn, PrintStream err){

	if ( null != filn)
	    this.ruleset = filn;
    }
    public boolean active(){ 

	return (null != this.ruleset);
    }

    public void init(){

	if (null == this.ruleset)

	    throw new IllegalArgumentException("Require ruleset.");

	else {

	    this.rules.setAttribute("org.yajre.semantic.java.bsh.Interpreter",this.interp);

	    File rulef = new File(this.ruleset);

	    if ( (!rulef.exists())||(!rulef.canRead()))

		throw new IllegalArgumentException("File not found `"+rulef.getAbsolutePath()+"'.");
	    else 
		try {
		    RuleLoader.load(this.rules,rulef);

		    this.model = this.rules.create();
		}
		catch ( Exception drx){

		    drx.printStackTrace();

		    throw new IllegalStateException("Rules initialization error.");
		}
	}
    }

    private final static String prompt = "rules> ";

    /**
     * Interactive Console UI.
     *
     * @param din Console input.
     * 
     * @param out Console output.
     * 
     * @param err Console error.
     */
    public final void console ( DataInputStream din, PrintStream out, PrintStream err) throws IOException {

	err.print(prompt);

	StringTokenizer strtok;

	String line, cmd, arg1, arg2s, list[];

	Object arg2;

	int cmdlen;

	while (null != (line = din.readLine())){
	    try {
		strtok = new StringTokenizer(line," \t");
	
		if ( strtok.hasMoreTokens()){

		    cmd = strtok.nextToken();

		    cmdlen = cmd.length();

		    if ( 1 == cmdlen){

			switch(cmd.charAt(0)){

			case 'a':
			case 'm':
			    if ( strtok.hasMoreTokens()){

				arg1 = strtok.nextToken();

				if ( strtok.hasMoreTokens()){

				    arg2s = strtok.nextToken();

				    /*
				     * special (`name|`name`) syntax:
				     *
				     *    lookup named value.
				     */
				    if ( arg2s.startsWith("`")){

					if ( arg2s.endsWith("`"))
					    arg2s = arg2s.substring(1,arg2s.length()-1);
					else
					    arg2s = arg2s.substring(1);

					try {
					    arg2 = this.interp.eval( (String)arg2s);
					}
					catch ( EvalError ever){

					    arg2 = null;

					    String msg = ever.getMessage();
					    try {
						int idx = msg.indexOf('>');
						
						if ( 0 < idx)
						    msg = msg.substring(idx+3).trim();
						
					    } catch ( Exception x){}
					    
					    out.println("[error] "+msg);
					}
				    }
				    else
					arg2 = arg2s;
				}
				else 
				    arg2 = null;

				if ( null == arg2){

				    if (cmd.equals("a"))

					this.model.assert(arg1,null);
				    else
					this.model.modify(arg1,null);

				    out.println("[null]");

				    break;
				}
				else {
				    if (cmd.equals("a"))

					this.model.assert(arg1,arg2);
				    else
					this.model.modify(arg1,arg2);

				    out.println(arg2);

				    break;
				}
			    }

			    System.err.println("Command 'assert' requires name and value arguments.");
			    break;

			case 'r':

			    if ( strtok.hasMoreTokens()){

				arg1 = strtok.nextToken();

				this.model.retract(arg1);
			    }
			    break;

			case 'f':

			    this.model.fireActions();

			    break;

			case 'l':

			    list = this.model.getActionsRuleNames();

			    if ( null == list)
				out.println("[none]");
			    else {
				for ( int many = list.length, cc = 0; cc < many; cc++){
				    out.println(list[cc]);
				}
			    }

			    break;

			case 'd':

			    if (null == this.debugger)

				this.debugger = new db( this.interp, din, out, err);

			    this.model.setChainListener(this.debugger);

			    break;

			case 'u':

			    this.model.unsetChainListener();

			    break;

			case 'q':

			    return;

			case '?':
			case 'h':
			    help(out);
			    break;

			default:
			    err.println("[unknown cmd]");
			    break;
			}
		    }
		    else 
			err.println("[unknown cmd]");
		}
	    }
	    catch ( Exception exc_in){

		err.print("[error] ");

		exc_in.printStackTrace(err);

		err.println();
	    }

	    err.print(prompt);
	}
	return;
    }

    private final static void usage( PrintStream out){
	out.println();
	out.println(" Usage: test -r <rules.xml>");
	out.println();
	out.println("    Rules testing console.  See also `bshi'.");
	out.println();
	out.println("    Reads simple command key language");
	out.println("    from input, type '?' for help.");
	out.println();
	out.println("    rules.xml         -- Rule set file.");
	out.println();
    }
    private final static void help( PrintStream out){
	out.println();
	out.println(" [Command]            -- [Description]");
	out.println();
	out.println(" a <name> <value>     -- Assert fact <name> = <value>.");
	out.println("                         Use (`value|`value`) to eval");
	out.println("                         the value using bsh.");
	out.println();
	out.println(" m <name> <value>     -- Modify fact <name> = <value>.");
	out.println("                         Use (`value|`value`) to eval");
	out.println("                         the value using bsh.");
	out.println();
	out.println(" r <name>             -- Retract fact <name>.");
	out.println();
	out.println(" f                    -- Fire rules.");
	out.println();
	out.println(" l                    -- List rules to be fired.");
	out.println();
	out.println(" d                    -- Enable rules debugger.");
	out.println();
	out.println(" u                    -- Disable rules debugger.");
	out.println();
	out.println(" q                    -- Quit (exit).");
	out.println();
	out.println(" ?|h                  -- This `help' message.");
	out.println();
    }
    public static void main(String[] argv){

	test tt = new test();

	try {
	    if ( null == argv)
		throw new IllegalArgumentException();
	    else {
		int alen = argv.length;
		if ( 1 > alen)
		    throw new IllegalArgumentException();
		else {
		    String arg;

		    for ( int argc = 0; argc < alen; argc++){
			arg = argv[argc];
			switch(arg.charAt(0)){

			case '-':
			case '/':
			    if ( 1 < arg.length()){
				switch(arg.charAt(1)){

				case 'r':
				    argc += 1;
				    if ( argc < alen){

					arg = argv[argc];

					tt.ruleset(arg,System.err);

					break;
				    }
				    else
					throw new IllegalArgumentException("Option `-r' requires argument.");

				default:
				    throw new IllegalArgumentException();
				}
			    }
			    else
				throw new IllegalArgumentException();
			    break;
			default:
			    throw new IllegalArgumentException();
			}
		    }
		}
	    }
	}
	catch ( IllegalArgumentException ilarg){
	    String msg = ilarg.getMessage();
	    if ( null == msg)
		usage(System.out);
	    else
		System.err.println("Error: "+msg);
	    System.exit(1);
	}

	/*
	 * Load and init engine
	 */
	try {
	    tt.init();

	    /*
	     * Enter console UI 
	     */
	    DataInputStream din = new DataInputStream(System.in);

	    tt.console(din,System.out,System.err);

	    System.err.println();
	    System.err.println("bye!");
	}
	catch ( IllegalArgumentException ilarg){
	    String msg = ilarg.getMessage();
	    if ( null == msg)
		usage(System.out);
	    else
		System.err.println("Error: "+msg);
	    System.exit(1);
	}
	catch ( Exception exc){

	    System.err.println();
	    System.err.println("Error loading rules.");
	    System.err.println();

	    System.exit(1);
	}
    }
}
